-------------------------------------------------------------------------------
-- $Id: emac.vhd,v 1.1 2007/12/11 06:41:03 sanjayk Exp $
-------------------------------------------------------------------------------
-- emac - entity/architecture pair
-------------------------------------------------------------------------------
--  ***************************************************************************
--  **  Copyright(C) 2006 by Xilinx, Inc. All rights reserved.               **
--  **                                                                       **
--  **  This text contains proprietary, confidential                         **
--  **  information of Xilinx, Inc. , is distributed by                      **
--  **  under license from Xilinx, Inc., and may be used,                    **
--  **  copied and/or disclosed only pursuant to the terms                   **
--  **  of a valid license agreement with Xilinx, Inc.                       **
--  **                                                                       **
--  **  Unmodified source code is guaranteed to place and route,             **
--  **  function and run at speed according to the datasheet                 **
--  **  specification. Source code is provided "as-is", with no              **
--  **  obligation on the part of Xilinx to provide support.                 **
--  **                                                                       **
--  **  Xilinx Hotline support of source code IP shall only include          **
--  **  standard level Xilinx Hotline support, and will only address         **
--  **  issues and questions related to the standard released Netlist        **
--  **  version of the core (and thus indirectly, the original core source). **
--  **                                                                       **
--  **  The Xilinx Support Hotline does not have access to source            **
--  **  code and therefore cannot answer specific questions related          **
--  **  to source HDL. The Xilinx Support Hotline will only be able          **
--  **  to confirm the problem in the Netlist version of the core.           **
--  **                                                                       **
--  **  This copyright and support notice must be retained as part           **
--  **  of this text at all times.                                           **
--  ***************************************************************************
--
-------------------------------------------------------------------------------
-- Filename:        emac.vhd
--
-- Description:     Design file for the Ethernet Lite MAC. 
-- VHDL-Standard:   VHDL'93
-------------------------------------------------------------------------------
-- Structure:   
--
--  xps_ethernet.vhd
--      \
--      \-- xemac.vhd
--           \
--           \-- xps_ipif_ssp1.vhd
--           \    
--           \
--           \-- emac_dpram.vhd                     
--           \    \                     
--           \    \-- RAMB16_S36_S36
--           \    \     or
--           \    \-- RAMB4_S8_S8
--           \                          
--           \-- emac.vhd                     
--                \                     
--                \-- receive.vhd
--                \      rx_state_2.vhd
--                \      rx_intrfce.vhd
--                \         veasyncfifo5w15ddstrb.edn
--                \            or
--                \         veasyncfifo5w15dbmem.edn
--                \            or
--                \         v2asyncfifo5w15ddstrb.edn
--                \            or
--                \         v2asyncfifo5w15dbmem.edn
--                \      cntr4bit.vhd
--                \      nibshiftin.vhd
--                \      nibshiftdain.vhd
--                \      cntr11bit.vhd
--                \      cntr12bit.vhd
--                \      dacheck.vhd
--                \      rxlengthcntr.vhd
--                \      crcgenrx.vhd
--                \      rxcrcnibcnt.vhd
--                \                     
--                \-- transmit_2.vhd
--                       crcgentx.vhd
--                          crcnibshiftreg
--                       tx_intrfce.vhd
--                          veasyncfifo5w15ddstrb.edn
--                             or
--                          veasyncfifo5w15dbmem.edn
--                             or
--                          v2asyncfifo5w15ddstrb.edn
--                             or
--                          v2asyncfifo5w15dbmem.edn
--                       cntr11bit.vhd
--                       nibshiftout.vhd
--                       tx_state_3.vhd
--                       deferral.vhd
--                          cntr5bit.vhd
--                          defer_state.vhd
--                       bocntr.vhd
--                          lfsr16.vhd
--                       msh_cnt.vhd
--                       ld_arith_reg.vhd
--
-------------------------------------------------------------------------------
-- Author:         MSH
-- History:
--  RSK            11/06/06
-- ^^^^^^
--                 First Version, Based on opb_ethernetlite v1.01.b
--  ~~~~~
-------------------------------------------------------------------------------
-- Naming Conventions:
--      active low signals:                     "*_n"
--      clock signals:                          "clk", "clk_div#", "clk_#x" 
--      reset signals:                          "rst", "rst_n" 
--      generics:                               "C_*" 
--      user defined types:                     "*_TYPE" 
--      state machine next state:               "*_ns" 
--      state machine current state:            "*_cs" 
--      combinatorial signals:                  "*_com" 
--      pipelined or register delay signals:    "*_d#" 
--      counter signals:                        "*cnt*"
--      clock enable signals:                   "*_ce" 
--      internal version of output port         "*_i"
--      device pins:                            "*_pin" 
--      ports:                                  - Names begin with Uppercase 
--      processes:                              "*_PROCESS" 
--      component instantiations:               "<ENTITY_>I_<#|FUNC>
-------------------------------------------------------------------------------
-- 
library ieee;
use ieee.STD_LOGIC_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

--Package file that contains constant definition for RESET_ACTIVE
library xps_ethernetlite_v2_00_c;
use xps_ethernetlite_v2_00_c.mac_pkg.all;
use xps_ethernetlite_v2_00_c.all;
library unisim;
use unisim.Vcomponents.all;
-------------------------------------------------------------------------------
-- Port Declaration
-------------------------------------------------------------------------------

entity emac is
  generic (
    C_DUPLEX       : integer    := 1; 
      -- 1 = full duplex, 0 = half duplex       
    NODE_MAC       : bit_vector := x"00005e00FACE";
    C_FAMILY       : string     := "virtex5"
    );
  port (  
    phy_tx_clk     : in std_logic;
    phy_rx_clk     : in std_logic;
    phy_crs        : in std_logic;
    phy_dv         : in std_logic;
    phy_rx_data    : in std_logic_vector (0 to 3);
    phy_col        : in std_logic;
    phy_rx_er      : in std_logic;
    
    phy_tx_en      : out std_logic;
    phy_tx_data    : out std_logic_vector (0 to 3);
    
    tx_DPM_ce      : out std_logic;
    tx_DPM_adr     : out std_logic_vector (0 to 11);
    tx_DPM_wr_data : out std_logic_vector (0 to 3);
    tx_DPM_rd_data : in  std_logic_vector (0 to 3);
    tx_DPM_wr_rd_n : out std_logic;
    tx_done        : out std_logic;
    tx_done_irq    : out std_logic;
    tx_pong_ping_l : in  std_logic;
    Tx_idle        : out std_logic;
    Rx_idle        : out std_logic;
    rx_DPM_ce      : out std_logic;
    rx_DPM_adr     : out std_logic_vector (0 to 11);
    rx_DPM_wr_data : out std_logic_vector (0 to 3);
    rx_DPM_rd_data : in  std_logic_vector (0 to 3);
    rx_DPM_wr_rd_n : out std_logic;
    rx_done        : out std_logic;
    rx_done_irq    : out std_logic;
    rx_pong_ping_l : in  std_logic;
            
    rst            : in  std_logic;
    clk            : in  std_logic
    );
end emac;

architecture imp of emac is

-------------------------------------------------------------------------------
--  Constant Declarations
-------------------------------------------------------------------------------

  constant STATUS_ADDR       : std_logic_vector(0 to 11) := x"ffe"; --x"ffe";
  constant FRMLEN_ADDR_ONE   : std_logic_vector(0 to 11) := x"fee"; --x"ffc";
  constant FRMLEN_ADDR_TWO   : std_logic_vector(0 to 11) := x"fef"; --x"ffd";  
  constant FRMLEN_ADDR_THREE : std_logic_vector(0 to 11) := x"fec"; --x"ffa";  
  constant FRMLEN_ADDR_FOUR  : std_logic_vector(0 to 11) := x"fed"; --x"ffb"; 
 
  signal phy_col_d1            : std_logic; -- added 3-03-05 MSH
  signal phy_crs_d1            : std_logic; -- added 3-03-05 MSH
  signal rxbuffer_addr         : std_logic_vector(0 to 11);
  signal rx_addr_load          : std_logic;
  signal rx_addr_en            : std_logic;
  signal rx_start              : std_logic;
  signal txbuffer_addr         : std_logic_vector(0 to 11);
  signal tx_addr_load          : std_logic;
  signal tx_addr_en            : std_logic;
  signal tx_start              : std_logic;
  signal tx_addr_const         : std_logic_vector(0 to 11);
  signal enbl_txLength_nib1    : std_logic;
  signal enbl_txLength_nib2    : std_logic;
  signal enbl_txLength_nib3    : std_logic;
  signal enbl_txLength_nib4    : std_logic;
  signal mac_addr_ram_addr     : std_logic_vector(0 to 3);
  signal mac_addr_ram_addr_rd  : std_logic_vector(0 to 3);
  signal mac_addr_ram_we       : std_logic;
  signal mac_addr_ram_addr_wr  : std_logic_vector(0 to 3);
  signal mac_addr_ram_data     : std_logic_vector(0 to 3);
  signal txClkEn               : std_logic;
  signal phy_tx_clk_d1         : std_logic;  
  signal phy_tx_clk_d2         : std_logic;
  signal mac_tx_frame_length   : std_logic_vector(0 to 15);
  signal adjustedPacketLength  : std_logic_vector(0 to 15);
  signal nibbleLength          : std_logic_vector(0 to 11);
  signal lengthDelay1          : std_logic;
  signal lengthDelay2          : std_logic;

-------------------------------------------------------------------------------
-- Component Declarations
-------------------------------------------------------------------------------

-- The following components are the building blocks of the EMAC

component FDR
  port (
    Q : out std_logic;
    C : in std_logic;
    D : in std_logic;
    R : in std_logic
    );
end component;

begin

  RX: entity xps_ethernetlite_v2_00_c.receive
    generic map
      (
      C_DUPLEX => C_DUPLEX,
      C_FAMILY => C_FAMILY   
      )    
    port map
      (   
      phy_rx_clk           => phy_rx_clk,
      phy_dv               => phy_dv,
      phy_rx_data          => phy_rx_data,
      phy_rx_col           => phy_col_d1, -- modified 3-03-05 MSH
      phy_rx_er            => phy_rx_er,
          
      rx_addr_load         => rx_addr_load,
      rx_addr_en           => rx_addr_en,
      rx_start             => rx_start,
      rx_done              => rx_done,
      rx_done_irq          => rx_done_irq,
      rx_pong_ping_l       => rx_pong_ping_l,
      rx_DPM_ce            => rx_DPM_ce,
      rx_DPM_wr_data       => rx_DPM_wr_data,
      rx_DPM_rd_data       => rx_DPM_rd_data,
      rx_DPM_wr_rd_n       => rx_DPM_wr_rd_n,
      Rx_idle              => Rx_idle,
      mac_addr_ram_addr_rd => mac_addr_ram_addr_rd,
      mac_addr_ram_data    => mac_addr_ram_data,

      rst                  => rst,
      clk                  => clk
      );      

  TX: entity xps_ethernetlite_v2_00_c.transmit
    generic map
      (
      C_DUPLEX => C_DUPLEX,
      C_FAMILY => C_FAMILY
      )
    port map
      (
      nibbleLength          =>  nibbleLength,
      txClkEn               =>  txClkEn,
      phy_tx_clk            =>  phy_tx_clk,
      phy_crs               =>  phy_crs_d1, -- modified 3-03-05 MSH
      phy_col               =>  phy_col_d1, -- modified 3-03-05 MSH
                   
      phy_tx_en             =>  phy_tx_en,
      phy_tx_data           =>  phy_tx_data,
      tx_addr_load          =>  tx_addr_load,
      tx_addr_en            =>  tx_addr_en,
      tx_start              =>  tx_start,
      tx_done               =>  tx_done,
      tx_done_irq           =>  tx_done_irq,
      tx_pong_ping_l        =>  tx_pong_ping_l,
      Tx_idle               =>  Tx_idle,
      enbl_txLength_nib1    =>  enbl_txLength_nib1,
      enbl_txLength_nib2    =>  enbl_txLength_nib2,
      enbl_txLength_nib3    =>  enbl_txLength_nib3,
      enbl_txLength_nib4    =>  enbl_txLength_nib4,
      lengthDelay1          => lengthDelay1,
      lengthDelay2          => lengthDelay2,
      tx_DPM_ce             =>  tx_DPM_ce,
      tx_DPM_wr_data        =>  tx_DPM_wr_data,
      tx_DPM_rd_data        => tx_DPM_rd_data,
      tx_DPM_wr_rd_n        => tx_DPM_wr_rd_n,
                   
      mac_addr_ram_we       => mac_addr_ram_we,
      mac_addr_ram_addr_wr  => mac_addr_ram_addr_wr,
      rst                   => rst,
      clk                   => clk
      );              

COLLISION_SYNC: FDR             -- added 3-03-05 MSH
  port map 
    (
    Q => phy_col_d1, --[out]
    C => clk, --[in]
    D => phy_col, --[in]
    R => rst --[in]
    );

C_SENSE_SYNC: FDR               -- added 3-03-05 MSH
  port map 
    (
    Q => phy_crs_d1, --[out]
    C => clk, --[in]
    D => phy_crs, --[in]
    R => rst --[in]
    );
             
  NODEMACADDRRAMI: entity xps_ethernetlite_v2_00_c.MacAddrRAM
     generic map 
       (
       MACAddr  => NODE_MAC
       )
     port map 
       (
       addr     => mac_addr_ram_addr,
       dout     => mac_addr_ram_data,
       din      => tx_DPM_rd_data,
       we       => mac_addr_ram_we,
       clk      => clk
       );

  mac_addr_ram_addr <= mac_addr_ram_addr_rd when mac_addr_ram_we = '0'
                  else mac_addr_ram_addr_wr;
---------------------------------------------------------------------------
-- RX Address Counter for the RxBuffer
---------------------------------------------------------------------------
  RXADDRCNT: process (clk)
    begin
      if clk'event and clk = '1' then
        if rst = '1' then
          rxbuffer_addr <= STATUS_ADDR;
        elsif rx_addr_load = '1' then
          rxbuffer_addr <= STATUS_ADDR;       
        elsif rx_start = '1' then
          rxbuffer_addr <= (others => '0');
        elsif rx_addr_en = '1' then
          rxbuffer_addr <= rxbuffer_addr + 1;
        end if;
      end if;
    end process RXADDRCNT;

  rx_DPM_adr <= rxbuffer_addr;
---------------------------------------------------------------------------
-- TX Address Counter for the TxBuffer (To Read)
---------------------------------------------------------------------------
  TXADDRCNT: process (clk)
    begin
      if clk'event and clk = '1' then
        if rst = '1' then
          txbuffer_addr <= tx_addr_const;
        elsif tx_addr_load = '1' then
          txbuffer_addr <= tx_addr_const;
        elsif tx_start = '1' then
          txbuffer_addr <= (others => '0');
        elsif tx_addr_en = '1' then
          txbuffer_addr <= txbuffer_addr + 1;
        end if;
      end if;
    end process TXADDRCNT;

  tx_addr_const <= FRMLEN_ADDR_ONE   when enbl_txLength_nib1 = '1' else
                   FRMLEN_ADDR_TWO   when enbl_txLength_nib2 = '1' else
                   FRMLEN_ADDR_THREE when enbl_txLength_nib3 = '1' else
                   FRMLEN_ADDR_FOUR  when enbl_txLength_nib4 = '1' else
                   STATUS_ADDR;  
  tx_DPM_adr <= txbuffer_addr;
  
-------------------------------------------------------------------------------
-- INT_tx_clk_sync_PROCESS
-------------------------------------------------------------------------------
-- This process syncronizes the tx clk and generates an enable pulse
-------------------------------------------------------------------------------
  
 INT_TX_CLK_SYNC_PROCESS : process (clk)
  begin  --   
    if (clk'event and clk = '1') then
      if (rst = RESET_ACTIVE) then
        phy_tx_clk_d1 <= '0';
        phy_tx_clk_d2 <= '0';
        txClkEn <= '0';
      else            
        phy_tx_clk_d1 <= phy_tx_clk;
        phy_tx_clk_d2 <= phy_tx_clk_d1;
        if (phy_tx_clk_d1 = '1' and phy_tx_clk_d2 = '0') then
          txClkEn <= '1';
        else
          txClkEn <= '0';
        end if;
      end if;
    end if;
  end process INT_TX_CLK_SYNC_PROCESS;  
  
  TX_LENGTH_REG : process (clk) -- delay from address to data valid
    begin
     if clk'event and clk = '1' then
       if (rst = RESET_ACTIVE) then
        mac_tx_frame_length <= (others => '0');
       elsif enbl_txLength_nib3 = '1' then
        mac_tx_frame_length(12 to 15) <= tx_DPM_rd_data;
        mac_tx_frame_length(0 to 11) <= (others => '0');
       elsif enbl_txLength_nib4 = '1' then
        mac_tx_frame_length(8 to 11) <= tx_DPM_rd_data;
        mac_tx_frame_length(0 to 7) <= (others => '0');
       elsif lengthDelay1 = '1' then
        mac_tx_frame_length(4 to 7) <= tx_DPM_rd_data;
        mac_tx_frame_length(0 to 3) <= (others => '0');
       elsif lengthDelay2 = '1' then
        mac_tx_frame_length(0 to 3) <= tx_DPM_rd_data;
       end if;
     end if;
  end process TX_LENGTH_REG;  
   
  ADJP : process(mac_tx_frame_length,adjustedPacketLength)
  begin
    if mac_tx_frame_length > MinimumPacketLength then
      adjustedPacketLength <= mac_tx_frame_length;
    else
      adjustedPacketLength <= MinimumPacketLength;
    end if;
    nibbleLength <= adjustedPacketLength(5 to 15) & '0';
  end process ADJP;    
  
end imp;
