-------------------------------------------------------------------------------
-- $Id: emac_dpram.vhd,v 1.1 2007/12/11 06:41:04 sanjayk Exp $
-------------------------------------------------------------------------------
-- emac_dpram.vhd  - entity/architecture pair
-------------------------------------------------------------------------------
--  ***************************************************************************
--  **  Copyright(C) 2006 by Xilinx, Inc. All rights reserved.               **
--  **                                                                       **
--  **  This text contains proprietary, confidential                         **
--  **  information of Xilinx, Inc. , is distributed by                      **
--  **  under license from Xilinx, Inc., and may be used,                    **
--  **  copied and/or disclosed only pursuant to the terms                   **
--  **  of a valid license agreement with Xilinx, Inc.                       **
--  **                                                                       **
--  **  Unmodified source code is guaranteed to place and route,             **
--  **  function and run at speed according to the datasheet                 **
--  **  specification. Source code is provided "as-is", with no              **
--  **  obligation on the part of Xilinx to provide support.                 **
--  **                                                                       **
--  **  Xilinx Hotline support of source code IP shall only include          **
--  **  standard level Xilinx Hotline support, and will only address         **
--  **  issues and questions related to the standard released Netlist        **
--  **  version of the core (and thus indirectly, the original core source). **
--  **                                                                       **
--  **  The Xilinx Support Hotline does not have access to source            **
--  **  code and therefore cannot answer specific questions related          **
--  **  to source HDL. The Xilinx Support Hotline will only be able          **
--  **  to confirm the problem in the Netlist version of the core.           **
--  **                                                                       **
--  **  This copyright and support notice must be retained as part           **
--  **  of this text at all times.                                           **
--  ***************************************************************************
--
-------------------------------------------------------------------------------
-- Filename:        emac_dpram.vhd
--
-- Description:     Realization of dprams   
--                  
-- VHDL-Standard:   VHDL'93
-------------------------------------------------------------------------------
-- Structure:   
--
--  xps_ethernetlite.vhd
--      \
--      \-- xemac.vhd
--           \
--           \--xps_ipif_ssp1.vhd
--           \
--           \-- emac_dpram.vhd                     
--           \    \                     
--           \    \-- RAMB16_S36_S36
--           \    \     or
--           \    \-- RAMB4_S8_S8
--           \                          
--           \-- emac.vhd                     
--                \                     
--                \                     
--                \-- receive.vhd
--                \      rx_state_2.vhd
--                \      rx_intrfce.vhd
--                \         veasyncfifo5w15ddstrb.edn
--                \            or
--                \         v2asyncfifo5w15ddstrb.edn
--                \      cntr4bit.vhd
--                \      nibshiftin.vhd
--                \      nibshiftdain.vhd
--                \      cntr11bit.vhd
--                \      cntr12bit.vhd
--                \      dacheck.vhd
--                \      rxlengthcntr.vhd
--                \      crcgenrx.vhd
--                \      rxcrcnibcnt.vhd
--                \                     
--                \-- transmit_2.vhd
--                       crcgentx.vhd
--                          crcnibshiftreg
--                       tx_intrfce.vhd
--                          veasyncfifo5w15ddstrb.edn
--                             or
--                          v2asyncfifo5w15ddstrb.edn
--                       cntr11bit.vhd
--                       nibshiftout.vhd
--                       tx_state_3.vhd
--                       deferral.vhd
--                          cntr5bit.vhd
--                          defer_state.vhd
--                       bocntr.vhd
--                          lfsr16.vhd
--                       msh_cnt.vhd
--                       ld_arith_reg.vhd
--
-------------------------------------------------------------------------------
-- Author:         MSH
-- History:
--  RSK            11/06/06
-- ^^^^^^
--                 First Version, Based on opb_ethernetlite v1.01.b
--  ~~~~~
-------------------------------------------------------------------------------
-- Naming Conventions:
--      active low signals:                     "*_n"
--      clock signals:                          "clk", "clk_div#", "clk_#x" 
--      reset signals:                          "rst", "rst_n" 
--      generics:                               "C_*" 
--      user defined types:                     "*_TYPE" 
--      state machine next state:               "*_ns" 
--      state machine current state:            "*_cs" 
--      combinatorial signals:                  "*_com" 
--      pipelined or register delay signals:    "*_d#" 
--      counter signals:                        "*cnt*"
--      clock enable signals:                   "*_ce" 
--      internal version of output port         "*_i"
--      device pins:                            "*_pin" 
--      ports:                                  - Names begin with Uppercase 
--      processes:                              "*_PROCESS" 
--      component instantiations:               "<ENTITY_>I_<#|FUNC>
-------------------------------------------------------------------------------
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;
use IEEE.std_logic_unsigned.all;

--Package file that contains constant definition for RESET_ACTIVE
library xps_ethernetlite_v2_00_c;
use xps_ethernetlite_v2_00_c.mac_pkg.all;
library proc_common_v2_00_a;
use proc_common_v2_00_a.all;
use proc_common_v2_00_a.family_support.all;

library unisim;
use unisim.Vcomponents.all;  -- uses BRAM primitives 

entity emac_dpram is
  generic 
   (
    C_FAMILY : string := "virtex5" 
    );
  port 
    (   
    -- a Port signals
    ce_a       : in  std_logic;
    wr_rd_n_a  : in  std_logic;
    adr_a      : in  std_logic_vector(0 to 11);
    data_in_a  : in  std_logic_vector(0 to 3);
    data_out_a : out std_logic_vector(0 to 3);
    
    -- b Port Signals
    ce_b       : in  std_logic;
    wr_rd_n_b  : in  std_logic;
    adr_b      : in  std_logic_vector(0 to 8);
    data_in_b  : in  std_logic_vector(0 to 31);
    data_out_b : out std_logic_vector(0 to 31);
    
    clk        : in std_logic;
    rst        : in std_logic
    );
end entity emac_dpram;

                
architecture imp of emac_dpram is

    component RAMB16_S4_S36
-- pragma translate_off
      generic
      (
        WRITE_MODE_A : string := "READ_FIRST"; 
      --WRITE_FIRST(default)/ READ_FIRST/ NO_CHANGE
        WRITE_MODE_B : string := "READ_FIRST" 
      --WRITE_FIRST(default)/ READ_FIRST/ NO_CHANGE
      );
-- pragma translate_on
      port (
        DOA   : out std_logic_vector (3 downto 0);
        DOB   : out std_logic_vector (31 downto 0);
        DOPB  : out std_logic_vector (3 downto 0);
        ADDRA : in std_logic_vector (11 downto 0);
        CLKA  : in std_logic;
        DIA   : in std_logic_vector (3 downto 0);
        ENA   : in std_logic;
        SSRA  : in std_logic;
        WEA   : in std_logic;
        ADDRB : in std_logic_vector (8 downto 0);
        CLKB  : in std_logic;
        DIB   : in std_logic_vector (31 downto 0);
        DIPB  : in std_logic_vector (3 downto 0);
        ENB   : in std_logic;
        SSRB  : in std_logic;
        WEB   : in std_logic
      );
    end component;
    
constant create_v2_mem  : boolean   := supported(C_FAMILY, u_RAMB16_S4_S36);  

-------------------------------------------------------------------------------
--  Signal and Type Declarations
-------------------------------------------------------------------------------

signal ce_a_i          : std_logic; -- added 03-03-05 MSH
signal ce_b_i          : std_logic; -- added 03-03-05 MSH
signal wr_rd_n_a_i     : std_logic; -- added 03-03-05 MSH
signal wr_rd_n_b_i     : std_logic; -- added 03-03-05 MSH

signal port_b_data_in  : STD_LOGIC_VECTOR (31 downto 0);
signal port_b_data_out : STD_LOGIC_VECTOR (31 downto 0);        

attribute WRITE_MODE_A : string;
attribute WRITE_MODE_A of I_DPB16_4_9: label is "READ_FIRST";
attribute WRITE_MODE_B : string;
attribute WRITE_MODE_B of I_DPB16_4_9: label is "READ_FIRST"; 
    
    
  
begin  -- architecture

  ce_a_i <= ce_a or rst; -- added 03-03-05 MSH
  ce_b_i <= ce_b or rst; -- added 03-03-05 MSH
  wr_rd_n_a_i <= wr_rd_n_a and not(rst); -- added 03-03-05 MSH
  wr_rd_n_b_i <= wr_rd_n_b and not(rst); -- added 03-03-05 MSH

-------------------------------------------------------------------------------
-- Using VII 4096 x 4 : 2048 x 8 Dual Port Primitive
-------------------------------------------------------------------------------
      port_b_data_in(31) <= data_in_b(0);
      port_b_data_in(30) <= data_in_b(1);
      port_b_data_in(29) <= data_in_b(2);
      port_b_data_in(28) <= data_in_b(3);
      port_b_data_in(27) <= data_in_b(4);
      port_b_data_in(26) <= data_in_b(5);
      port_b_data_in(25) <= data_in_b(6);
      port_b_data_in(24) <= data_in_b(7);
      port_b_data_in(23) <= data_in_b(8);
      port_b_data_in(22) <= data_in_b(9);
      port_b_data_in(21) <= data_in_b(10);
      port_b_data_in(20) <= data_in_b(11);
      port_b_data_in(19) <= data_in_b(12);
      port_b_data_in(18) <= data_in_b(13);
      port_b_data_in(17) <= data_in_b(14);
      port_b_data_in(16) <= data_in_b(15);
      port_b_data_in(15) <= data_in_b(16);
      port_b_data_in(14) <= data_in_b(17);
      port_b_data_in(13) <= data_in_b(18);
      port_b_data_in(12) <= data_in_b(19);
      port_b_data_in(11) <= data_in_b(20);
      port_b_data_in(10) <= data_in_b(21);
      port_b_data_in(9)  <= data_in_b(22);
      port_b_data_in(8)  <= data_in_b(23);
      port_b_data_in(7)  <= data_in_b(24);
      port_b_data_in(6)  <= data_in_b(25);
      port_b_data_in(5)  <= data_in_b(26);
      port_b_data_in(4)  <= data_in_b(27);
      port_b_data_in(3)  <= data_in_b(28);
      port_b_data_in(2)  <= data_in_b(29);
      port_b_data_in(1)  <= data_in_b(30);
      port_b_data_in(0)  <= data_in_b(31);

      data_out_b(31) <= port_b_data_out(0);
      data_out_b(30) <= port_b_data_out(1);
      data_out_b(29) <= port_b_data_out(2);
      data_out_b(28) <= port_b_data_out(3);
      data_out_b(27) <= port_b_data_out(4);
      data_out_b(26) <= port_b_data_out(5);
      data_out_b(25) <= port_b_data_out(6);
      data_out_b(24) <= port_b_data_out(7);  
      data_out_b(23) <= port_b_data_out(8);
      data_out_b(22) <= port_b_data_out(9);
      data_out_b(21) <= port_b_data_out(10);
      data_out_b(20) <= port_b_data_out(11);
      data_out_b(19) <= port_b_data_out(12);
      data_out_b(18) <= port_b_data_out(13);
      data_out_b(17) <= port_b_data_out(14);
      data_out_b(16) <= port_b_data_out(15);  
      data_out_b(15) <= port_b_data_out(16);
      data_out_b(14) <= port_b_data_out(17);
      data_out_b(13) <= port_b_data_out(18);
      data_out_b(12) <= port_b_data_out(19);
      data_out_b(11) <= port_b_data_out(20);
      data_out_b(10) <= port_b_data_out(21);
      data_out_b(9)  <= port_b_data_out(22);
      data_out_b(8)  <= port_b_data_out(23);  
      data_out_b(7)  <= port_b_data_out(24);
      data_out_b(6)  <= port_b_data_out(25);
      data_out_b(5)  <= port_b_data_out(26);
      data_out_b(4)  <= port_b_data_out(27);
      data_out_b(3)  <= port_b_data_out(28);
      data_out_b(2)  <= port_b_data_out(29);
      data_out_b(1)  <= port_b_data_out(30);
      data_out_b(0)  <= port_b_data_out(31);  
      
      I_DPB16_4_9: RAMB16_S4_S36
        port map (
          DOA   => data_out_a,      --[out]
          DOB   => port_b_data_out, --[out]
          DOPB  => open,            --[out]
          ADDRA => adr_a,           --[in]
          CLKA  => clk,             --[in]
          DIA   => data_in_a,       --[in]
          ENA   => ce_a_i,          --[in]
          SSRA  => rst,             --[in]
          WEA   => wr_rd_n_a_i,     --[in]
          ADDRB => adr_b,           --[in]
          CLKB  => clk,             --[in]
          DIB   => port_b_data_in,  --[in]
          DIPB  => (others => '0'), --[in]
          ENB   => ce_b_i,          --[in]
          SSRB  => rst,             --[in]
          WEB   => wr_rd_n_b_i      --[in]
        );

assert (create_v2_mem)
report "The primitive RAMB16_S4_S36 is not Supported by the Target device"
severity FAILURE; 
   
end architecture imp;
