-------------------------------------------------------------------------------
-- $Id: nibshiftin.vhd,v 1.1 2007/12/11 06:41:10 sanjayk Exp $
-------------------------------------------------------------------------------
-- nibshiftin - entity/architecture pair
-------------------------------------------------------------------------------
--  ***************************************************************************
--  **  Copyright(C) 2006 by Xilinx, Inc. All rights reserved.               **
--  **                                                                       **
--  **  This text contains proprietary, confidential                         **
--  **  information of Xilinx, Inc. , is distributed by                      **
--  **  under license from Xilinx, Inc., and may be used,                    **
--  **  copied and/or disclosed only pursuant to the terms                   **
--  **  of a valid license agreement with Xilinx, Inc.                       **
--  **                                                                       **
--  **  Unmodified source code is guaranteed to place and route,             **
--  **  function and run at speed according to the datasheet                 **
--  **  specification. Source code is provided "as-is", with no              **
--  **  obligation on the part of Xilinx to provide support.                 **
--  **                                                                       **
--  **  Xilinx Hotline support of source code IP shall only include          **
--  **  standard level Xilinx Hotline support, and will only address         **
--  **  issues and questions related to the standard released Netlist        **
--  **  version of the core (and thus indirectly, the original core source). **
--  **                                                                       **
--  **  The Xilinx Support Hotline does not have access to source            **
--  **  code and therefore cannot answer specific questions related          **
--  **  to source HDL. The Xilinx Support Hotline will only be able          **
--  **  to confirm the problem in the Net list version of the core.           **
--  **                                                                       **
--  **  This copyright and support notice must be retained as part           **
--  **  of this text at all times.                                           **
--  ***************************************************************************
--
-------------------------------------------------------------------------------
-- Filename:        nibshiftin.vhd
-- 
-- Description:     
--                  This file contains the a 32 bit register that can shift by 
--                  nibbles
-- VHDL-Standard:   VHDL'93
-------------------------------------------------------------------------------
-- Structure:   
--
--  xps_ethernet.vhd
--      \
--      \-- xemac.vhd
--           \
--           \-- xps_ipif_ssp1.vhd
--           \    
--           \
--           \-- emac_dpram.vhd                     
--           \    \                     
--           \    \-- RAMB16_S36_S36
--           \    \     or
--           \    \-- RAMB4_S8_S8
--           \                          
--           \-- emac.vhd                     
--                \                     
--                \-- receive.vhd
--                \      rx_state_2.vhd
--                \      rx_intrfce.vhd
--                \         veasyncfifo5w15ddstrb.edn
--                \            or
--                \         veasyncfifo5w15dbmem.edn
--                \            or
--                \         v2asyncfifo5w15ddstrb.edn
--                \            or
--                \         v2asyncfifo5w15dbmem.edn
--                \      cntr4bit.vhd
--                \      nibshiftin.vhd
--                \      nibshiftdain.vhd
--                \      cntr11bit.vhd
--                \      cntr12bit.vhd
--                \      dacheck.vhd
--                \      rxlengthcntr.vhd
--                \      crcgenrx.vhd
--                \      rxcrcnibcnt.vhd
--                \                     
--                \-- transmit_2.vhd
--                       crcgentx.vhd
--                          crcnibshiftreg
--                       tx_intrfce.vhd
--                          veasyncfifo5w15ddstrb.edn
--                             or
--                          veasyncfifo5w15dbmem.edn
--                             or
--                          v2asyncfifo5w15ddstrb.edn
--                             or
--                          v2asyncfifo5w15dbmem.edn
--                       cntr11bit.vhd
--                       nibshiftout.vhd
--                       tx_state_3.vhd
--                       deferral.vhd
--                          cntr5bit.vhd
--                          defer_state.vhd
--                       bocntr.vhd
--                          lfsr16.vhd
--                       msh_cnt.vhd
--                       ld_arith_reg.vhd
--
-------------------------------------------------------------------------------
-- Author:         M. Scott Hurt
-- History:
--  RSK            11/06/06
-- ^^^^^^
--                 First Version, Based on opb_ethernetlite v1.01.b
--  ~~~~~
-------------------------------------------------------------------------------
-- Naming Conventions:
--      active low signals:                     "*_n"
--      clock signals:                          "clk", "clk_div#", "clk_#x" 
--      reset signals:                          "rst", "rst_n" 
--      generics:                               "C_*" 
--      user defined types:                     "*_TYPE" 
--      state machine next state:               "*_ns" 
--      state machine current state:            "*_cs" 
--      combinatorial signals:                  "*_com" 
--      pipelined or register delay signals:    "*_d#" 
--      counter signals:                        "*cnt*"
--      clock enable signals:                   "*_ce" 
--      internal version of output port         "*_i"
--      device pins:                            "*_pin" 
--      ports:                                  - Names begin with Uppercase 
--      processes:                              "*_PROCESS" 
--      component instantiations:               "<ENTITY_>I_<#|FUNC>
-------------------------------------------------------------------------------
-- 
library ieee;
use ieee.std_logic_1164.all;

--Package file that contains constant definition for RESET_ACTIVE
library xps_ethernetlite_v2_00_c;
use xps_ethernetlite_v2_00_c.all;

-------------------------------------------------------------------------------
-- Port Declaration
-------------------------------------------------------------------------------

entity nibshiftin is

  port 
       (
        Clk   : in  std_logic; 
        Rst   : in  std_logic;
        Din   : in  std_logic_vector(0 to 3);
        Shift : in  std_logic; 
        Nibcnt: in  std_logic_vector(0 to 2); --
        Dout  : out std_logic_vector(0 to 31)
        );
end nibshiftin;


architecture implementation of nibshiftin is

-------------------------------------------------------------------------------
--  Constant Declarations
-------------------------------------------------------------------------------
-- Constants used in this design are found in mac_pkg.vhd
-------------------------------------------------------------------------------
-- Signal and Type Declarations
-------------------------------------------------------------------------------

signal nibData  : std_logic_vector (0 to 31);
  
begin
 -------------------------------------------------------------------------------
 -- INT_reg_PROCESS
 -------------------------------------------------------------------------------
 -- 
 -------------------------------------------------------------------------------

  INT_REG_PROCESS : process (Clk)
  begin  --
    if (Clk'event and Clk = '1') then
      if Rst = '1' then
        nibData <= (others => '0');
      elsif (Shift = '1') then
        case Nibcnt is
          when "000" =>
            nibData(4 to 7)   <= Din;
            nibData(0 to 3)   <= nibData(0 to 3);
            nibData(12 to 15) <= nibData(12 to 15);
            nibData(8 to 11)  <= nibData(8 to 11);
            nibData(20 to 23) <= nibData(20 to 23);
            nibData(16 to 19) <= nibData(16 to 19);
            nibData(28 to 31) <= nibData(28 to 31);
            nibData(24 to 27) <= nibData(24 to 27);
          when "001" =>
            nibData(4 to 7)   <= nibData(4 to 7);
            nibData(0 to 3)   <= Din;
            nibData(12 to 15) <= nibData(12 to 15);
            nibData(8 to 11)  <= nibData(8 to 11);
            nibData(20 to 23) <= nibData(20 to 23);
            nibData(16 to 19) <= nibData(16 to 19);
            nibData(28 to 31) <= nibData(28 to 31);
            nibData(24 to 27) <= nibData(24 to 27);
          when "010" =>
            nibData(4 to 7)   <= nibData(4 to 7);
            nibData(0 to 3)   <= nibData(0 to 3);
            nibData(12 to 15) <= Din;
            nibData(8 to 11)  <= nibData(8 to 11);
            nibData(20 to 23) <= nibData(20 to 23);
            nibData(16 to 19) <= nibData(16 to 19);
            nibData(28 to 31) <= nibData(28 to 31);
            nibData(24 to 27) <= nibData(24 to 27);            
          when "011" =>
            nibData(4 to 7)   <= nibData(4 to 7);
            nibData(0 to 3)   <= nibData(0 to 3);
            nibData(12 to 15) <= nibData(12 to 15);
            nibData(8 to 11)  <= Din;
            nibData(20 to 23) <= nibData(20 to 23);
            nibData(16 to 19) <= nibData(16 to 19);
            nibData(28 to 31) <= nibData(28 to 31);
            nibData(24 to 27) <= nibData(24 to 27);            
          when "100" =>
            nibData(4 to 7)   <= nibData(4 to 7);
            nibData(0 to 3)   <= nibData(0 to 3);
            nibData(12 to 15) <= nibData(12 to 15);
            nibData(8 to 11)  <= nibData(8 to 11);
            nibData(20 to 23) <= Din;
            nibData(16 to 19) <= nibData(16 to 19);
            nibData(28 to 31) <= nibData(28 to 31);
            nibData(24 to 27) <= nibData(24 to 27);            
          when "101" =>
            nibData(4 to 7)   <= nibData(4 to 7);
            nibData(0 to 3)   <= nibData(0 to 3);
            nibData(12 to 15) <= nibData(12 to 15);
            nibData(8 to 11)  <= nibData(8 to 11);
            nibData(20 to 23) <= nibData(20 to 23);
            nibData(16 to 19) <= Din;
            nibData(28 to 31) <= nibData(28 to 31);
            nibData(24 to 27) <= nibData(24 to 27);            
          when "110" =>
            nibData(4 to 7)   <= nibData(4 to 7);
            nibData(0 to 3)   <= nibData(0 to 3);
            nibData(12 to 15) <= nibData(12 to 15);
            nibData(8 to 11)  <= nibData(8 to 11);
            nibData(20 to 23) <= nibData(20 to 23);
            nibData(16 to 19) <= nibData(16 to 19);
            nibData(28 to 31) <= Din;
            nibData(24 to 27) <= nibData(24 to 27);            
          when "111" =>
            nibData(4 to 7)   <= nibData(4 to 7);
            nibData(0 to 3)   <= nibData(0 to 3);
            nibData(12 to 15) <= nibData(12 to 15);
            nibData(8 to 11)  <= nibData(8 to 11);
            nibData(20 to 23) <= nibData(20 to 23);
            nibData(16 to 19) <= nibData(16 to 19);
            nibData(28 to 31) <= nibData(28 to 31);
            nibData(24 to 27) <= Din;            
          when others         => null;
            nibData <= nibData;       
    end case;            
    
      else
       null;
      end if;
    end if;
  end process INT_REG_PROCESS;

  Dout <= nibData;

  end implementation;

