-------------------------------------------------------------------------------
-- $Id: receive.vhd,v 1.1 2007/12/11 06:41:12 sanjayk Exp $
-------------------------------------------------------------------------------
-- receive - entity/architecture pair
-------------------------------------------------------------------------------
--  ***************************************************************************
--  **  Copyright(C) 2006 by Xilinx, Inc. All rights reserved.               **
--  **                                                                       **
--  **  This text contains proprietary, confidential                         **
--  **  information of Xilinx, Inc. , is distributed by                      **
--  **  under license from Xilinx, Inc., and may be used,                    **
--  **  copied and/or disclosed only pursuant to the terms                   **
--  **  of a valid license agreement with Xilinx, Inc.                       **
--  **                                                                       **
--  **  Unmodified source code is guaranteed to place and route,             **
--  **  function and run at speed according to the datasheet                 **
--  **  specification. Source code is provided "as-is", with no              **
--  **  obligation on the part of Xilinx to provide support.                 **
--  **                                                                       **
--  **  Xilinx Hotline support of source code IP shall only include          **
--  **  standard level Xilinx Hotline support, and will only address         **
--  **  issues and questions related to the standard released Netlist        **
--  **  version of the core (and thus indirectly, the original core source). **
--  **                                                                       **
--  **  The Xilinx Support Hotline does not have access to source            **
--  **  code and therefore cannot answer specific questions related          **
--  **  to source HDL. The Xilinx Support Hotline will only be able          **
--  **  to confirm the problem in the Netlist version of the core.           **
--  **                                                                       **
--  **  This copyright and support notice must be retained as part           **
--  **  of this text at all times.                                           **
--  ***************************************************************************
--
-------------------------------------------------------------------------------
-- Filename:        receive.vhd
--
-- Description:     This is the receive path portion of the design
-- VHDL-Standard:   VHDL'93
-------------------------------------------------------------------------------
-- Structure:   
--
--  xps_ethernetlite.vhd
--      \
--      \-- xemac.vhd
--           \
--           \--xps_ipif_ssp1.vhd
--           \
--           \-- emac_dpram.vhd                     
--           \    \                     
--           \    \-- RAMB16_S36_S36
--           \    \     or
--           \    \-- RAMB4_S8_S8
--           \                          
--           \-- emac.vhd                     
--                \                     
--                \                     
--                \-- receive.vhd
--                \      rx_state_2.vhd
--                \      rx_intrfce.vhd
--                \         veasyncfifo5w15ddstrb.edn
--                \            or
--                \         v2asyncfifo5w15ddstrb.edn
--                \      cntr4bit.vhd
--                \      nibshiftin.vhd
--                \      nibshiftdain.vhd
--                \      cntr11bit.vhd
--                \      cntr12bit.vhd
--                \      dacheck.vhd
--                \      rxlengthcntr.vhd
--                \      crcgenrx.vhd
--                \      rxcrcnibcnt.vhd
--                \                     
--                \-- transmit_2.vhd
--                       crcgentx.vhd
--                          crcnibshiftreg
--                       tx_intrfce.vhd
--                          veasyncfifo5w15ddstrb.edn
--                             or
--                          v2asyncfifo5w15ddstrb.edn
--                       cntr11bit.vhd
--                       nibshiftout.vhd
--                       tx_state_3.vhd
--                       deferral.vhd
--                          cntr5bit.vhd
--                          defer_state.vhd
--                       bocntr.vhd
--                          lfsr16.vhd
--                       msh_cnt.vhd
--                       ld_arith_reg.vhd
--
-------------------------------------------------------------------------------
-- Author:         MSH
-- History:
--  RSK            11/06/06
-- ^^^^^^
--                 First Version, Based on opb_ethernetlite v1.01.b
--  ~~~~~
-------------------------------------------------------------------------------
-- Naming Conventions:
--      active low signals:                     "*_n"
--      clock signals:                          "clk", "clk_div#", "clk_#x" 
--      reset signals:                          "rst", "rst_n" 
--      generics:                               "C_*" 
--      user defined types:                     "*_TYPE" 
--      state machine next state:               "*_ns" 
--      state machine current state:            "*_cs" 
--      combinatorial signals:                  "*_com" 
--      pipelined or register delay signals:    "*_d#" 
--      counter signals:                        "*cnt*"
--      clock enable signals:                   "*_ce" 
--      internal version of output port         "*_i"
--      device pins:                            "*_pin" 
--      ports:                                  - Names begin with Uppercase 
--      processes:                              "*_PROCESS" 
--      component instantiations:               "<ENTITY_>I_<#|FUNC>
-------------------------------------------------------------------------------
-- 
library ieee;
use ieee.STD_LOGIC_1164.all;

--Package file that contains constant definition for RESET_ACTIVE
library xps_ethernetlite_v2_00_c;
use xps_ethernetlite_v2_00_c.all;

library proc_common_v2_00_a;
use proc_common_v2_00_a.all;
-- synopsys translate_off
Library XilinxCoreLib;
library unisim;
library simprim;
-- synopsys translate_on

-------------------------------------------------------------------------------
-- Port Declaration
-------------------------------------------------------------------------------

entity receive is
  generic 
    (
    C_DUPLEX : integer := 1; 
      -- 1 = full duplex, 0 = half duplex    
    C_FAMILY             : string  := "virtex5"  
    );
  port 
    (
    phy_rx_clk           : in std_logic;
    phy_dv               : in std_logic;
    phy_rx_data          : in std_logic_vector (0 to 3);
    phy_rx_col           : in std_logic;
    phy_rx_er            : in std_logic;

    rx_addr_load         : out std_logic;
    rx_addr_en           : out std_logic;
    rx_start             : out std_logic;
    rx_done              : out std_logic;
    rx_done_irq          : out std_logic;
    rx_pong_ping_l       : in  std_logic;
    rx_DPM_ce            : out std_logic;
    rx_DPM_wr_data       : out std_logic_vector (0 to 3);
    rx_DPM_rd_data       : in std_logic_vector (0 to 3);    
    rx_DPM_wr_rd_n       : out std_logic;
    Rx_idle              : out std_logic;
    mac_addr_ram_addr_rd : out std_logic_vector(0 to 3);
    mac_addr_ram_data    : in std_logic_vector (0 to 3);
    
    rst                  : in std_logic;
    clk                  : in std_logic
    );
end receive;

-------------------------------------------------------------------------------
-- Definition of Generics:
--          No Generics were used for this Entity.
-------------------------------------------------------------------------------

architecture imp of receive is

-------------------------------------------------------------------------------
--  Constant Declarations
-------------------------------------------------------------------------------


-------------------------------------------------------------------------------
--  Signal and Type Declarations
-------------------------------------------------------------------------------
signal fifo_empty                    : std_logic;
signal fifo_full                     : std_logic;
signal emac_rx_rd                    : std_logic;
signal emac_rx_rd_data               : std_logic_vector(0 to 5);
signal emac_rx_rd_data_d1            : std_logic_vector(0 to 5); -- 03-26-04
signal rxAbortRst                    : std_logic;

signal rxChannelReset                : std_logic;

signal rxBusFifoRdAck                : std_logic;

signal rxComboCrcRst                 : std_logic;
signal rxComboCrcEn                  : std_logic;
signal crcOk                         : std_logic;
signal rxCrcRst                      : std_logic;
signal rxCrcEn                       : std_logic;
signal rxCrcEn_d1                    : std_logic; -- 03-26-04

signal receive_enable                : std_logic; -- 03-26-04
signal fifo_reset                    : std_logic; -- 03-26-04

begin

-------------------------------------------------------------------------------
-- rx control state machine
-------------------------------------------------------------------------------
  inst_rx_state: entity xps_ethernetlite_v2_00_c.rx_state
    generic map (
      C_DUPLEX             => C_DUPLEX
      )  
    port map (
      emac_rx_rd_data_d1     => emac_rx_rd_data_d1, -- 03-26-04
      receive_enable         => receive_enable, -- 03-26-04
      rxBusFifoRdAck         => rxBusFifoRdAck,
      busFifoEmpty           => fifo_empty,
      collision              => phy_rx_col,
      dataValid              => emac_rx_rd_data(4),
      rxError                => emac_rx_rd_data(5),
      busFifoData            => emac_rx_rd_data(0 to 3),
      crcOk                  => crcOk,
      busFifoRd              => emac_rx_rd,
      rxAbortRst             => rxAbortRst,
      rxCrcRst               => rxCrcRst,
      rxCrcEn                => rxCrcEn,
      rx_addr_load           => rx_addr_load,
      rx_addr_en             => rx_addr_en,
      rx_start               => rx_start,
      rx_done                => rx_done,
      rx_done_irq            => rx_done_irq,
      rx_pong_ping_l         => rx_pong_ping_l,
      rx_DPM_ce              => rx_DPM_ce,
      rx_DPM_wr_data         => rx_DPM_wr_data,
      rx_DPM_rd_data         => rx_DPM_rd_data,
      rx_DPM_wr_rd_n         => rx_DPM_wr_rd_n,
      Rx_idle                => Rx_idle,
      mac_addr_ram_addr_rd   => mac_addr_ram_addr_rd,
      mac_addr_ram_data      => mac_addr_ram_data,
      clk                    => clk,
      rst                    => rxChannelReset);      

  rxChannelReset <= rst;
  
-------------------------------------------------------------------------------
-- rx interface contains the ethernet rx fifo
-------------------------------------------------------------------------------

  inst_rx_intrfce: entity xps_ethernetlite_v2_00_c.rx_intrfce
    port map (    
      phy_rx_clk      => phy_rx_clk,
      phy_tx_clk      => '0',
      internalWrapEn  => '0',
      phy_rx_er       => phy_rx_er,
      phy_dv          => phy_dv,
      phy_tx_en       => '0',             
      phy_rx_data     => phy_rx_data,
      phy_tx_data     => x"0",
      rcv_en          => receive_enable, -- 03-26-04
      fifo_empty      => fifo_empty,
      fifo_full       => fifo_full,
      emac_rx_rd      => emac_rx_rd,
      emac_rx_rd_data => emac_rx_rd_data,
      rdAck           => rxBusFifoRdAck,
      rst             => fifo_reset, -- 03-26-04
      clk             => clk);    
      
  fifo_reset <= rst or not(receive_enable); -- 03-26-04
-------------------------------------------------------------------------------
-- crc checker
-------------------------------------------------------------------------------
  inst_crcgenrx: entity xps_ethernetlite_v2_00_c.crcgenrx
    port map(
      rst     => rxComboCrcRst,
      clk     => clk,
      data    => emac_rx_rd_data(0 to 3),
      dataEn  => rxComboCrcEn,
      crcOk   => crcOk);

  rxComboCrcRst <= rst or rxCrcRst or rxAbortRst;
  rxComboCrcEn <=  rxCrcEn_d1;                 
  
  REG_PROCESS : process (clk)
  begin  --
    if (clk'event and clk = '1') then     -- rising clock edge
      if (rst = '1') then
        emac_rx_rd_data_d1 <= "000000";
        rxCrcEn_d1 <= '0';
      else
        emac_rx_rd_data_d1 <= emac_rx_rd_data;   
        rxCrcEn_d1 <= rxCrcEn;
      end if;
    end if;
  end process REG_PROCESS;

end imp;
