-------------------------------------------------------------------------------
-- $Id: rx_intrfce.vhd,v 1.1 2007/12/11 06:41:12 sanjayk Exp $
-------------------------------------------------------------------------------
-- rx_intrfce - entity/architecture pair
-------------------------------------------------------------------------------
--  ***************************************************************************
--  **  Copyright(C) 2006 by Xilinx, Inc. All rights reserved.               **
--  **                                                                       **
--  **  This text contains proprietary, confidential                         **
--  **  information of Xilinx, Inc. , is distributed by                      **
--  **  under license from Xilinx, Inc., and may be used,                    **
--  **  copied and/or disclosed only pursuant to the terms                   **
--  **  of a valid license agreement with Xilinx, Inc.                       **
--  **                                                                       **
--  **  Unmodified source code is guaranteed to place and route,             **
--  **  function and run at speed according to the datasheet                 **
--  **  specification. Source code is provided "as-is", with no              **
--  **  obligation on the part of Xilinx to provide support.                 **
--  **                                                                       **
--  **  Xilinx Hotline support of source code IP shall only include          **
--  **  standard level Xilinx Hotline support, and will only address         **
--  **  issues and questions related to the standard released Netlist        **
--  **  version of the core (and thus indirectly, the original core source). **
--  **                                                                       **
--  **  The Xilinx Support Hotline does not have access to source            **
--  **  code and therefore cannot answer specific questions related          **
--  **  to source HDL. The Xilinx Support Hotline will only be able          **
--  **  to confirm the problem in the Netlist version of the core.           **
--  **                                                                       **
--  **  This copyright and support notice must be retained as part           **
--  **  of this text at all times.                                           **
--  ***************************************************************************
--
-------------------------------------------------------------------------------
-- Filename:        rx_intrfce.vhd
--
-- Description:     This is the ethernet receive interface. 
-- VHDL-Standard:   VHDL'93
-------------------------------------------------------------------------------
-- Structure:   
--
--  xps_ethernetlite.vhd
--      \
--      \-- xemac.vhd
--           \
--           \--xps_ipif_ssp1.vhd
--           \
--           \-- emac_dpram.vhd                     
--           \    \                     
--           \    \-- RAMB16_S36_S36
--           \    \     or
--           \    \-- RAMB4_S8_S8
--           \                          
--           \-- emac.vhd                     
--                \                     
--                \                     
--                \-- receive.vhd
--                \      rx_state_2.vhd
--                \      rx_intrfce.vhd
--                \         veasyncfifo5w15ddstrb.edn
--                \            or
--                \         v2asyncfifo5w15ddstrb.edn
--                \      cntr4bit.vhd
--                \      nibshiftin.vhd
--                \      nibshiftdain.vhd
--                \      cntr11bit.vhd
--                \      cntr12bit.vhd
--                \      dacheck.vhd
--                \      rxlengthcntr.vhd
--                \      crcgenrx.vhd
--                \      rxcrcnibcnt.vhd
--                \                     
--                \-- transmit_2.vhd
--                       crcgentx.vhd
--                          crcnibshiftreg
--                       tx_intrfce.vhd
--                          veasyncfifo5w15ddstrb.edn
--                             or
--                          v2asyncfifo5w15ddstrb.edn
--                       cntr11bit.vhd
--                       nibshiftout.vhd
--                       tx_state_3.vhd
--                       deferral.vhd
--                          cntr5bit.vhd
--                          defer_state.vhd
--                       bocntr.vhd
--                          lfsr16.vhd
--                       msh_cnt.vhd
--                       ld_arith_reg.vhd
--
-------------------------------------------------------------------------------
-- Author:         MSH
-- History:
--  RSK            11/06/06
-- ^^^^^^
--                 First Version, Based on opb_ethernetlite v1.01.b
--  ~~~~~
-------------------------------------------------------------------------------
-- Naming Conventions:
--      active low signals:                     "*_n"
--      clock signals:                          "clk", "clk_div#", "clk_#x" 
--      reset signals:                          "rst", "rst_n" 
--      generics:                               "C_*" 
--      user defined types:                     "*_TYPE" 
--      state machine next state:               "*_ns" 
--      state machine current state:            "*_cs" 
--      combinatorial signals:                  "*_com" 
--      pipelined or register delay signals:    "*_d#" 
--      counter signals:                        "*cnt*"
--      clock enable signals:                   "*_ce" 
--      internal version of output port         "*_i"
--      device pins:                            "*_pin" 
--      ports:                                  - Names begin with Uppercase 
--      processes:                              "*_PROCESS" 
--      component instantiations:               "<ENTITY_>I_<#|FUNC>
-------------------------------------------------------------------------------
-- 
library ieee;
use ieee.STD_LOGIC_1164.all;

--Package file that contains constant definition for RESET_ACTIVE
library xps_ethernetlite_v2_00_c;
use xps_ethernetlite_v2_00_c.all;
library proc_common_v2_00_a;
use proc_common_v2_00_a.all;
use proc_common_v2_00_a.family.all;
-- synopsys translate_off
Library XilinxCoreLib;
library unisim;
library simprim;
-- synopsys translate_on

-------------------------------------------------------------------------------
-- Port Declaration
-------------------------------------------------------------------------------

entity rx_intrfce is
  port (
    Phy_rx_clk      : in std_logic;
    Phy_tx_clk      : in std_logic;
    InternalWrapEn  : in std_logic;
    Phy_rx_er       : in std_logic;
    Phy_dv          : in std_logic;
    Phy_tx_en       : in std_logic;         
    Phy_rx_data     : in std_logic_vector (0 to 3);
    Phy_tx_data     : in std_logic_vector (0 to 3);
    Rcv_en          : in std_logic;
    Fifo_empty      : out std_logic;
    Fifo_full       : out std_logic;
    Emac_rx_rd      : in std_logic;
    Emac_rx_rd_data : out std_logic_vector (0 to 5);
    RdAck           : out std_logic;
    Rst             : in std_logic;
    Clk             : in std_logic
    );
end rx_intrfce;

-------------------------------------------------------------------------------
-- Definition of Generics:
--          No Generics were used for this Entity.
--
-- Definition of Ports:
--         
-------------------------------------------------------------------------------

architecture implementation of rx_intrfce is

-------------------------------------------------------------------------------
--  Constant Declarations
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
--  Signal and Type Declarations
-------------------------------------------------------------------------------

signal rxBusCombo      : std_logic_vector (0 to 5);
signal txBusCombo      : std_logic_vector (0 to 5);

signal rx_wr_en        : std_logic;
signal rx_data         : std_logic_vector (0 to 5);
signal rx_fifo_full    : std_logic;
signal rx_fifo_empty   : std_logic;
signal rx_rd_ack       : std_logic;
signal wrap_wr_en      : std_logic;
signal wrap_data       : std_logic_vector (0 to 5);
signal wrap_fifo_full  : std_logic;
signal wrap_fifo_empty : std_logic;
signal wrap_rd_ack     : std_logic;

-------------------------------------------------------------------------------
-- Component Declarations
-------------------------------------------------------------------------------
-- The following components are the building blocks of the EMAC
-------------------------------------------------------------------------------

-- Start Configuration Specification
  
-- End Configuration Specification
 
-- This was copied from the COREGEN ethernetlite_v1_00_a_dmem_v2.vho file
  component ethernetlite_v1_01_b_dmem_v2
    port (
      din   : IN std_logic_VECTOR(5 downto 0);
      wr_en : IN std_logic;
      wr_clk: IN std_logic;
      rd_en : IN std_logic;
      rd_clk: IN std_logic;
      ainit : IN std_logic;
      dout  : OUT std_logic_VECTOR(5 downto 0);
      full  : OUT std_logic;
      empty : OUT std_logic;
      rd_ack: OUT std_logic);
  end component;

-- synopsys translate_off
  for all : ethernetlite_v1_01_b_dmem_v2 use entity XilinxCoreLib.async_fifo_v5_1(behavioral)
    generic map(
      c_use_blockmem    => 0,
      c_rd_count_width  => 2,
      c_has_wr_ack      => 0,
      c_has_almost_full => 0,
      c_has_wr_err      => 0,
      c_wr_err_low      => 0,
      c_wr_ack_low      => 0,
      c_data_width      => 6,
      c_enable_rlocs    => 0,
      c_rd_err_low      => 0,
      c_wr_count_width  => 2,
      c_rd_ack_low      => 0,
      c_has_rd_count    => 0,
      c_has_almost_empty=> 0,
      c_has_rd_ack      => 1,
      c_has_wr_count    => 0,
      c_fifo_depth      => 15,
      c_has_rd_err      => 0);

-- synopsys translate_on
                            
-- Synplicity black box declaration
attribute box_type : string;
attribute box_type of ethernetlite_v1_01_b_dmem_v2: component is "black_box";

begin
-- Connect the Coregen fifo to the VHDL wrapper
       I_RX_FIFO : ethernetlite_v1_01_b_dmem_v2
         port map (
           din    => rxBusCombo,
           wr_en  => rx_wr_en,
           wr_clk => Phy_rx_clk,
           rd_en  => Emac_rx_rd,
           rd_clk => Clk,
           ainit  => Rst,
           dout   => rx_data,
           full   => rx_fifo_full,
           empty  => rx_fifo_empty,      
           rd_ack => rx_rd_ack);    
         
rxBusCombo      <= (Phy_rx_data & Phy_dv & Phy_rx_er);
Emac_rx_rd_data <= rx_data; 
RdAck           <= rx_rd_ack; 
Fifo_full       <= rx_fifo_full; 
Fifo_empty      <= rx_fifo_empty; 
rx_wr_en        <= Rcv_en; 
wrap_wr_en      <= '0'; 

           
end implementation;
