-------------------------------------------------------------------------------
-- $Id: tx_intrfce.vhd,v 1.1 2007/12/11 06:41:16 sanjayk Exp $
-------------------------------------------------------------------------------
-- tx_intrfce - entity/architecture pair
-------------------------------------------------------------------------------
--  ***************************************************************************
--  **  Copyright(C) 2006 by Xilinx, Inc. All rights reserved.               **
--  **                                                                       **
--  **  This text contains proprietary, confidential                         **
--  **  information of Xilinx, Inc. , is distributed by                      **
--  **  under license from Xilinx, Inc., and may be used,                    **
--  **  copied and/or disclosed only pursuant to the terms                   **
--  **  of a valid license agreement with Xilinx, Inc.                       **
--  **                                                                       **
--  **  Unmodified source code is guaranteed to place and route,             **
--  **  function and run at speed according to the datasheet                 **
--  **  specification. Source code is provided "as-is", with no              **
--  **  obligation on the part of Xilinx to provide support.                 **
--  **                                                                       **
--  **  Xilinx Hotline support of source code IP shall only include          **
--  **  standard level Xilinx Hotline support, and will only address         **
--  **  issues and questions related to the standard released Netlist        **
--  **  version of the core (and thus indirectly, the original core source). **
--  **                                                                       **
--  **  The Xilinx Support Hotline does not have access to source            **
--  **  code and therefore cannot answer specific questions related          **
--  **  to source HDL. The Xilinx Support Hotline will only be able          **
--  **  to confirm the problem in the Netlist version of the core.           **
--  **                                                                       **
--  **  This copyright and support notice must be retained as part           **
--  **  of this text at all times.                                           **
--  ***************************************************************************
--
-------------------------------------------------------------------------------
-- Filename:        tx_intrfce.vhd
--
-- Description:     This is the ethernet transmit interface. 
-- VHDL-Standard:   VHDL'93
-------------------------------------------------------------------------------
-- Structure:   
--
--  xps_ethernetlite.vhd
--      \
--      \-- xemac.vhd
--           \
--           \--xps_ipif_ssp1.vhd
--           \
--           \-- emac_dpram.vhd                     
--           \    \                     
--           \    \-- RAMB16_S36_S36
--           \    \     or
--           \    \-- RAMB4_S8_S8
--           \                          
--           \-- emac.vhd                     
--                \                     
--                \                     
--                \-- receive.vhd
--                \      rx_state_2.vhd
--                \      rx_intrfce.vhd
--                \         veasyncfifo5w15ddstrb.edn
--                \            or
--                \         v2asyncfifo5w15ddstrb.edn
--                \      cntr4bit.vhd
--                \      nibshiftin.vhd
--                \      nibshiftdain.vhd
--                \      cntr11bit.vhd
--                \      cntr12bit.vhd
--                \      dacheck.vhd
--                \      rxlengthcntr.vhd
--                \      crcgenrx.vhd
--                \      rxcrcnibcnt.vhd
--                \                     
--                \-- transmit_2.vhd
--                       crcgentx.vhd
--                          crcnibshiftreg
--                       tx_intrfce.vhd
--                          veasyncfifo5w15ddstrb.edn
--                             or
--                          v2asyncfifo5w15ddstrb.edn
--                       cntr11bit.vhd
--                       nibshiftout.vhd
--                       tx_state_3.vhd
--                       deferral.vhd
--                          cntr5bit.vhd
--                          defer_state.vhd
--                       bocntr.vhd
--                          lfsr16.vhd
--                       msh_cnt.vhd
--                       ld_arith_reg.vhd
--
-------------------------------------------------------------------------------
-- Author:         MSH
-- History:
--  RSK            11/06/06
-- ^^^^^^
--                 First Version, Based on opb_ethernetlite v1.01.b
--  ~~~~~
-------------------------------------------------------------------------------
-- Naming Conventions:
--      active low signals:                     "*_n"
--      clock signals:                          "clk", "clk_div#", "clk_#x" 
--      reset signals:                          "rst", "rst_n" 
--      generics:                               "C_*" 
--      user defined types:                     "*_TYPE" 
--      state machine next state:               "*_ns" 
--      state machine current state:            "*_cs" 
--      combinatorial signals:                  "*_com" 
--      pipelined or register delay signals:    "*_d#" 
--      counter signals:                        "*cnt*"
--      clock enable signals:                   "*_ce" 
--      internal version of output port         "*_i"
--      device pins:                            "*_pin" 
--      ports:                                  - Names begin with Uppercase 
--      processes:                              "*_PROCESS" 
--      component instantiations:               "<ENTITY_>I_<#|FUNC>
-------------------------------------------------------------------------------
-- 
library ieee;
use ieee.STD_LOGIC_1164.all;

--Package file that contains constant definition for RESET_ACTIVE
library xps_ethernetlite_v2_00_c;
use xps_ethernetlite_v2_00_c.all;
library proc_common_v2_00_a;
use proc_common_v2_00_a.all;
use proc_common_v2_00_a.family.all;
-- synopsys translate_off
Library XilinxCoreLib;
library simprim;
-- synopsys translate_on
library unisim;
use unisim.Vcomponents.all;

-------------------------------------------------------------------------------
-- Port Declaration
-------------------------------------------------------------------------------

entity tx_intrfce is

  port 
    (
    phy_tx_clk        : in std_logic;
    emac_tx_wr_data   : in std_logic_vector (0 to 3);
    tx_er             : in std_logic;
    phyTxEn           : in std_logic;
    tx_en             : in std_logic;
    emac_tx_wr        : in std_logic;
    rst               : in std_logic;
    clk               : in std_logic;
    fifo_rd_ack       : out std_logic;
    fifo_empty        : out std_logic;
    fifo_almost_empty : out std_logic;
    fifo_full         : out std_logic;
    phy_tx_data       : out std_logic_vector (0 to 5)
    );
end tx_intrfce;

architecture implementation of tx_intrfce is

-------------------------------------------------------------------------------
--  Constant Declarations
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
--  Signal and Type Declarations
-------------------------------------------------------------------------------

signal bus_combo    : std_logic_vector (0 to 5);
signal fifo_empty_i : std_logic;
-------------------------------------------------------------------------------
-- Component Declarations
-------------------------------------------------------------------------------
-- The following components are the building blocks of the EMAC
-------------------------------------------------------------------------------
component FDR
  port 
   (
    Q : out std_logic;
    C : in std_logic;
    D : in std_logic;
    R : in std_logic
   );
end component;
 
-------------------------------------------------------------------------------
-- COREGEN asynchronous FIFO that is 5 wide and 15 deep
-------------------------------------------------------------------------------
 
-- Start Configuration Specification
  
-- End Configuration Specification
-- This was copied from the COREGEN ethernetlite_v1_00_a_dmem_v2.vho file
  component ethernetlite_v1_01_b_dmem_v2
    port 
      (
      din   : in std_logic_vector(5 downto 0);
      wr_en : in std_logic;
      wr_clk: in std_logic;
      rd_en : in std_logic;
      rd_clk: in std_logic;
      ainit : in std_logic;
      dout  : out std_logic_vector(5 downto 0);
      full  : out std_logic;
      empty : out std_logic;
      rd_ack: out std_logic
      );
  end component;

-- synopsys translate_off
  for all : ethernetlite_v1_01_b_dmem_v2 
            use entity XilinxCoreLib.async_fifo_v5_1(behavioral)
    generic map
     (
      c_wr_count_width   => 2,
      c_has_rd_err       => 0,
      c_data_width       => 6,
      c_has_almost_full  => 0,
      c_rd_err_low       => 0,
      c_has_wr_ack       => 0,
      c_wr_ack_low       => 0,
      c_fifo_depth       => 15,
      c_rd_count_width   => 2,
      c_has_wr_err       => 0,
      c_has_almost_empty => 0,
      c_rd_ack_low       => 0,
      c_has_wr_count     => 0,
      c_use_blockmem     => 0,
      c_has_rd_ack       => 1,
      c_has_rd_count     => 0,
      c_wr_err_low       => 0,
      c_enable_rlocs     => 0
      );

-- synopsys translate_on
                            
-- Synplicity black box declaration
 attribute box_type : string;
 attribute box_type of ethernetlite_v1_01_b_dmem_v2: component is "black_box";

begin
    
pipeIt: FDR
  port map
   (
    Q => fifo_empty,   --[out]
    C => clk,          --[in]
    D => fifo_empty_i, --[in]
    R => rst           --[in]
   );
     

-- Connect the Coregen fifo to the VHDL wrapper
       I_TX_FIFO : ethernetlite_v1_01_b_dmem_v2
         port map 
           (
           din    => bus_combo,
           wr_en  => emac_tx_wr,
           wr_clk => clk,
           rd_en  => tx_en,
           rd_clk => phy_tx_clk,
           ainit  => rst,
           dout   => phy_tx_data,
           full   => fifo_full,
           empty  => fifo_empty_i,    
           rd_ack => open
           );    

  bus_combo <= (emac_tx_wr_data & tx_er & phyTxEn); 
           
end implementation;

