-------------------------------------------------------------------------------
-- $Id: tx_state_3.vhd,v 1.1 2007/12/11 06:41:16 sanjayk Exp $
-------------------------------------------------------------------------------
-- tx_state_3 - entity/architecture pair
-------------------------------------------------------------------------------
--  ***************************************************************************
--  **  Copyright(C) 2006 by Xilinx, Inc. All rights reserved.               **
--  **                                                                       **
--  **  This text contains proprietary, confidential                         **
--  **  information of Xilinx, Inc. , is distributed by                      **
--  **  under license from Xilinx, Inc., and may be used,                    **
--  **  copied and/or disclosed only pursuant to the terms                   **
--  **  of a valid license agreement with Xilinx, Inc.                       **
--  **                                                                       **
--  **  Unmodified source code is guaranteed to place and route,             **
--  **  function and run at speed according to the datasheet                 **
--  **  specification. Source code is provided "as-is", with no              **
--  **  obligation on the part of Xilinx to provide support.                 **
--  **                                                                       **
--  **  Xilinx Hotline support of source code IP shall only include          **
--  **  standard level Xilinx Hotline support, and will only address         **
--  **  issues and questions related to the standard released Netlist        **
--  **  version of the core (and thus indirectly, the original core source). **
--  **                                                                       **
--  **  The Xilinx Support Hotline does not have access to source            **
--  **  code and therefore cannot answer specific questions related          **
--  **  to source HDL. The Xilinx Support Hotline will only be able          **
--  **  to confirm the problem in the Netlist version of the core.           **
--  **                                                                       **
--  **  This copyright and support notice must be retained as part           **
--  **  of this text at all times.                                           **
--  ***************************************************************************
--
-------------------------------------------------------------------------------
-- Filename:        tx_state_3.vhd
-- 
-- Description:     
--                  This file contains the transmit control state machine.
-- VHDL-Standard:   VHDL'93
-------------------------------------------------------------------------------
-- Structure:   
--
--  xps_ethernetlite.vhd
--      \
--      \-- xemac.vhd
--           \
--           \--xps_ipif_ssp1.vhd
--           \
--           \-- emac_dpram.vhd                     
--           \    \                     
--           \    \-- RAMB16_S36_S36
--           \    \     or
--           \    \-- RAMB4_S8_S8
--           \                          
--           \-- emac.vhd                     
--                \                     
--                \                     
--                \-- receive.vhd
--                \      rx_state_2.vhd
--                \      rx_intrfce.vhd
--                \         veasyncfifo5w15ddstrb.edn
--                \            or
--                \         v2asyncfifo5w15ddstrb.edn
--                \      cntr4bit.vhd
--                \      nibshiftin.vhd
--                \      nibshiftdain.vhd
--                \      cntr11bit.vhd
--                \      cntr12bit.vhd
--                \      dacheck.vhd
--                \      rxlengthcntr.vhd
--                \      crcgenrx.vhd
--                \      rxcrcnibcnt.vhd
--                \                     
--                \-- transmit_2.vhd
--                       crcgentx.vhd
--                          crcnibshiftreg
--                       tx_intrfce.vhd
--                          veasyncfifo5w15ddstrb.edn
--                             or
--                          v2asyncfifo5w15ddstrb.edn
--                       cntr11bit.vhd
--                       nibshiftout.vhd
--                       tx_state_3.vhd
--                       deferral.vhd
--                          cntr5bit.vhd
--                          defer_state.vhd
--                       bocntr.vhd
--                          lfsr16.vhd
--                       msh_cnt.vhd
--                       ld_arith_reg.vhd
--
-------------------------------------------------------------------------------
-- Author:         MSH
-- History:
--  RSK            11/06/06
-- ^^^^^^
--                 First Version, Based on opb_ethernetlite v1.01.b
--  ~~~~~
--  RSK            12/15/06
-- ^^^^^^
--                 The outputs mac_addr_ram_addr_wr and mac_addr_ram_we are 
--                 registered to improve the performance
--  ~~~~~
--  RSK            1/31/07
-- ^^^^^^
--                 removed "elsif (txie = '1')" in line 1477 to address FV issue
--  ~~~~~
-------------------------------------------------------------------------------
-- Naming Conventions:
--      active low signals:                     "*_n"
--      clock signals:                          "clk", "clk_div#", "clk_#x" 
--      reset signals:                          "rst", "rst_n" 
--      generics:                               "C_*" 
--      user defined types:                     "*_TYPE" 
--      state machine next state:               "*_ns" 
--      state machine current state:            "*_cs" 
--      combinatorial signals:                  "*_com" 
--      pipelined or register delay signals:    "*_d#" 
--      counter signals:                        "*cnt*"
--      clock enable signals:                   "*_ce" 
--      internal version of output port         "*_i"
--      device pins:                            "*_pin" 
--      ports:                                  - Names begin with Uppercase 
--      processes:                              "*_PROCESS" 
--      component instantiations:               "<ENTITY_>I_<#|FUNC>
-------------------------------------------------------------------------------
-- 
library ieee;
use ieee.STD_LOGIC_1164.all;

--Package file that contains constant definition for RESET_ACTIVE
library xps_ethernetlite_v2_00_c;
use xps_ethernetlite_v2_00_c.all;

-- synopsys translate_off
Library XilinxCoreLib;
library simprim;
-- synopsys translate_on
library unisim;
use unisim.Vcomponents.all;
-------------------------------------------------------------------------------
-- Port Declaration
-------------------------------------------------------------------------------

entity tx_state_3 is
  generic 
     (
       C_DUPLEX : integer := 1
       -- 1 = full duplex, 0 = half duplex   
     );
  port 
    (
    txClkEn              : in std_logic; 
    jam_rst              : out std_logic;
    txRst                : in std_logic;
    deferring            : in std_logic;        
    colRetryCnt          : in std_logic_vector (0 to 4);
    colWindowNibCnt      : in std_logic_vector (0 to 7);
    jamTxNibCnt          : in std_logic_vector (0 to 3);
    txNibbleCnt          : in std_logic_vector (0 to 11);
    busFifoWrNibbleCnt   : in std_logic_vector (0 to 11);
    crcCnt               : in std_logic_vector (0 to 3);
    busFifoFull          : in std_logic;
    busFifoEmpty         : in std_logic;
    phyCollision         : in std_logic;
    initBackoff          : out std_logic;
    txRetryRst           : out std_logic;
    txExcessDefrlRst     : out std_logic;
    txLateColnRst        : out std_logic;          
    txColRetryCntRst_n   : out std_logic;
    txColRetryCntEnbl    : out std_logic;
    jamTxNibCntEnbl      : out std_logic;
    txNibbleCntRst       : out std_logic;
    txEnNibbleCnt        : out std_logic;
    txNibbleCntLd        : out std_logic;
    busFifoWrCntRst      : out std_logic;
    busFifoWrCntEn       : out std_logic;
    enblPre              : out std_logic;
    enblSFD              : out std_logic;
    enblData             : out std_logic;
    enblJam              : out std_logic;
    enblCRC              : out std_logic;
    busFifoWr            : out std_logic;
    phytx_en             : out std_logic;
    txCrcEn              : out std_logic;
    txCrcShftOutEn       : out std_logic;
    tx_addr_load         : out std_logic;
    tx_addr_en           : out std_logic;
    tx_start             : out std_logic;
    tx_done              : out std_logic;
    tx_done_irq          : out std_logic;
    tx_pong_ping_l       : in  std_logic;
    Tx_idle              : out std_logic;
    enbl_txLength_nib1   : out std_logic;
    enbl_txLength_nib2   : out std_logic;
    enbl_txLength_nib3   : out std_logic;
    enbl_txLength_nib4   : out std_logic;
    lengthDelay1         : out std_logic;
    lengthDelay2         : out std_logic;
    tx_DPM_ce            : out std_logic;
    tx_DPM_wr_data       : out std_logic_vector (0 to 3);
    tx_DPM_rd_data       : in  std_logic_vector (0 to 3);
    tx_DPM_wr_rd_n       : out std_logic;
    enblclear            : out std_logic;
    
    mac_addr_ram_we      : out std_logic;
    mac_addr_ram_addr_wr : out std_logic_vector(0 to 3);      
    pre_sfd_done         : out std_logic;
    clk                  : in std_logic;
    rst                  : in std_logic
    );
 
end tx_state_3;

-------------------------------------------------------------------------------
-- Definition of Generics:
--          No Generics were used for this Entity.
--
-- Definition of Ports:
--         
-------------------------------------------------------------------------------

architecture implementation of tx_state_3 is
-------------------------------------------------------------------------------
--  Constant Declarations
-------------------------------------------------------------------------------
-- Constants used in this design are found in mac_pkg.vhd
-------------------------------------------------------------------------------
-- Signal and Type Declarations
-------------------------------------------------------------------------------
  signal idle                         : std_logic; -- state  0
  signal txLngthRdNib1                : std_logic; -- state  1
  signal txLngthRdNib2                : std_logic; -- state  2
  signal txLngthRdNib3                : std_logic; -- state  3
  signal txLngthRdNib4                : std_logic; -- state  4
  signal lngthDelay1                  : std_logic; -- state  5
  signal lngthDelay2                  : std_logic; -- state  6
  signal ldLngthCntr                  : std_logic; -- state  7
  signal preamble                     : std_logic; -- state  8
  signal checkBusFifoFullSFD          : std_logic; -- state  9
  signal SFD                          : std_logic; -- state 10
  signal checkBusFifoFull             : std_logic; -- state 11
  signal loadBusFifo                  : std_logic; -- state 12
  signal CheckCrc                     : std_logic; -- state 13  
  signal checkBusFifoFullCrc          : std_logic; -- state 14
  signal loadBusFifoCrc               : std_logic; -- state 15
  signal waitFifoEmpty                : std_logic; -- state 16
  signal txDone                       : std_logic; -- state 17
  signal checkBusFifoFullJam          : std_logic; -- state 18
  signal loadBusFifoJam               : std_logic; -- state 19
  signal half_dup_error               : std_logic; -- state 20
  signal collisionRetry               : std_logic; -- state 21
  signal retryWaitFifoEmpty           : std_logic; -- state 22
  signal retryReset                   : std_logic; -- state 23
  signal txDone2                      : std_logic; -- state 24
  signal txDonePause                  : std_logic; -- state 25
  signal chgMacAdr1                   : std_logic; -- state 26
  signal chgMacAdr2                   : std_logic; -- state 27
  signal chgMacAdr3                   : std_logic; -- state 28
  signal chgMacAdr4                   : std_logic; -- state 29
  signal chgMacAdr5                   : std_logic; -- state 30
  signal chgMacAdr6                   : std_logic; -- state 31
  signal chgMacAdr7                   : std_logic; -- state 32
  signal chgMacAdr8                   : std_logic; -- state 33
  signal chgMacAdr9                   : std_logic; -- state 34
  signal chgMacAdr10                  : std_logic; -- state 35
  signal chgMacAdr11                  : std_logic; -- state 36
  signal chgMacAdr12                  : std_logic; -- state 37
  signal chgMacAdr13                  : std_logic; -- state 38
  signal chgMacAdr14                  : std_logic; -- state 39
  
  signal idle_D                       : std_logic; -- state  0
  signal txLngthRdNib1_D              : std_logic; -- state  1
  signal txLngthRdNib2_D              : std_logic; -- state  2
  signal txLngthRdNib3_D              : std_logic; -- state  3
  signal txLngthRdNib4_D              : std_logic; -- state  4
  signal lngthDelay1_D                : std_logic; -- state  5
  signal lngthDelay2_D                : std_logic; -- state  6
  signal ldLngthCntr_D                : std_logic; -- state  7
  signal preamble_D                   : std_logic; -- state  8
  signal checkBusFifoFullSFD_D        : std_logic; -- state  9
  signal SFD_D                        : std_logic; -- state 10
  signal checkBusFifoFull_D           : std_logic; -- state 11
  signal loadBusFifo_D                : std_logic; -- state 12
  signal CheckCrc_D                   : std_logic; -- state 13  
  signal checkBusFifoFullCrc_D        : std_logic; -- state 14
  signal loadBusFifoCrc_D             : std_logic; -- state 15
  signal waitFifoEmpty_D              : std_logic; -- state 16
  signal txDone_D                     : std_logic; -- state 17
  signal checkBusFifoFullJam_D        : std_logic; -- state 18
  signal loadBusFifoJam_D             : std_logic; -- state 19
  signal half_dup_error_D             : std_logic; -- state 20
  signal collisionRetry_D             : std_logic; -- state 21
  signal retryWaitFifoEmpty_D         : std_logic; -- state 22
  signal retryReset_D                 : std_logic; -- state 23
  signal txDone2_D                    : std_logic; -- state 24
  signal txDonePause_D                : std_logic; -- state 25
  signal chgMacAdr1_D                 : std_logic; -- state 26
  signal chgMacAdr2_D                 : std_logic; -- state 27
  signal chgMacAdr3_D                 : std_logic; -- state 28
  signal chgMacAdr4_D                 : std_logic; -- state 29
  signal chgMacAdr5_D                 : std_logic; -- state 30
  signal chgMacAdr6_D                 : std_logic; -- state 31
  signal chgMacAdr7_D                 : std_logic; -- state 32
  signal chgMacAdr8_D                 : std_logic; -- state 33
  signal chgMacAdr9_D                 : std_logic; -- state 34
  signal chgMacAdr10_D                : std_logic; -- state 35
  signal chgMacAdr11_D                : std_logic; -- state 36
  signal chgMacAdr12_D                : std_logic; -- state 37
  signal chgMacAdr13_D                : std_logic; -- state 38
  signal chgMacAdr14_D                : std_logic; -- state 39
  
  signal txNibbleCntRst_i             : std_logic; 
  signal txEnNibbleCnt_i              : std_logic;
  signal txNibbleCntLd_i              : std_logic;
  signal busFifoWr_i                  : std_logic;
  signal phytx_en_i                   : std_logic;
  signal phytx_en_i_n                 : std_logic;
  signal txCrcEn_i                    : std_logic;
  signal retrying                     : std_logic; 
  signal retrying_i                   : std_logic; 
  signal phytx_en_reg                 : std_logic;
  signal busFifoWrCntRst_reg          : std_logic;
  signal retrying_reg                 : std_logic;
  signal txCrcEn_reg                  : std_logic;
  signal busFifoWrCntRst_i            : std_logic;
  signal state_machine_rst            : std_logic;
  signal full_half_n                  : std_logic;
  
  signal goto_idle                    : std_logic; -- state  0
  signal stay_idle                    : std_logic; -- state  0
  signal goto_txLngthRdNib1_1         : std_logic; -- state  1
  signal goto_txLngthRdNib1_2         : std_logic; -- state  1
  signal goto_txLngthRdNib2           : std_logic; -- state  2
  signal goto_txLngthRdNib3           : std_logic; -- state  3
  signal goto_txLngthRdNib4           : std_logic; -- state  4
  signal goto_lngthDelay1             : std_logic; -- state  5
  signal goto_lngthDelay2             : std_logic; -- state  6
  signal goto_ldLngthCntr             : std_logic; -- state  7
  signal stay_ldLngthCntr             : std_logic; -- state  7
  signal goto_preamble                : std_logic; -- state  8
  signal stay_preamble                : std_logic; -- state  8
  signal goto_checkBusFifoFullSFD     : std_logic; -- state  9 
  signal stay_checkBusFifoFullSFD     : std_logic; -- state  9
  signal goto_SFD                     : std_logic; -- state 10 
  signal stay_SFD                     : std_logic; -- state 10 
  signal goto_checkBusFifoFull_1      : std_logic; -- state 11 
  signal goto_checkBusFifoFull_2      : std_logic; -- state 11
  signal stay_checkBusFifoFull        : std_logic; -- state 11
  signal goto_loadBusFifo             : std_logic; -- state 12
  signal goto_checkCrc                : std_logic; -- state 13 
  signal goto_checkBusFifoFullCrc_1   : std_logic; -- state 14
  signal goto_checkBusFifoFullCrc_2   : std_logic; -- state 14
  signal stay_checkBusFifoFullCrc     : std_logic; -- state 14
  signal goto_loadBusFifoCrc_1        : std_logic; -- state 15
  signal goto_waitFifoEmpty_2         : std_logic; -- state 16
  signal stay_waitFifoEmpty           : std_logic; -- state 16
  signal goto_txDone_1                : std_logic; -- state 17
  signal goto_txDone_2                : std_logic; -- state 17
  signal goto_checkBusFifoFullJam_1   : std_logic; -- state 18
  signal goto_checkBusFifoFullJam_2   : std_logic; -- state 18
  signal stay_checkBusFifoFullJam     : std_logic; -- state 18
  signal goto_loadBusFifoJam          : std_logic; -- state 19
  signal goto_half_dup_error_1        : std_logic; -- state 20
  signal goto_half_dup_error_2        : std_logic; -- state 20
  signal goto_collisionRetry          : std_logic; -- state 21
  signal goto_retryWaitFifoEmpty      : std_logic; -- state 22
  signal stay_retryWaitFifoEmpty      : std_logic; -- state 22
  signal goto_retryReset              : std_logic; -- state 23
  signal goto_txDone2                 : std_logic; -- state 24
  signal goto_txDonePause             : std_logic; -- state 25
  signal goto_chgMacAdr1              : std_logic; -- state 26
  signal goto_chgMacAdr2              : std_logic; -- state 27
  signal goto_chgMacAdr3              : std_logic; -- state 28
  signal goto_chgMacAdr4              : std_logic; -- state 29
  signal goto_chgMacAdr5              : std_logic; -- state 30
  signal goto_chgMacAdr6              : std_logic; -- state 31
  signal goto_chgMacAdr7              : std_logic; -- state 32
  signal goto_chgMacAdr8              : std_logic; -- state 33
  signal goto_chgMacAdr9              : std_logic; -- state 34
  signal goto_chgMacAdr10             : std_logic; -- state 35
  signal goto_chgMacAdr11             : std_logic; -- state 36
  signal goto_chgMacAdr12             : std_logic; -- state 37
  signal goto_chgMacAdr13             : std_logic; -- state 38
  signal goto_chgMacAdr14             : std_logic; -- state 39
  
  signal txNibbleCnt_is_1             : std_logic;
  signal busFifoWrNibbleCnt_is_14     : std_logic;
  signal busFifoWrNibbleCnt_not_14    : std_logic;   
  signal busFifoWrNibbleCnt_is_15     : std_logic;
  signal busFifoWrNibbleCnt_not_15    : std_logic;
  signal crcCnt_not_0                 : std_logic;
  signal crcCnt_is_0                  : std_logic;
  signal jamTxNibCnt_not_0            : std_logic;
  signal jamTxNibCnt_is_0             : std_logic;
  signal colWindowNibCnt_not_0        : std_logic;
  signal colWindowNibCnt_is_0         : std_logic;
  signal colRetryCnt_is_15            : std_logic;
  signal pre_SFD_zero                 : std_logic;
  signal waitdone_pre_sfd             : std_logic;
  signal txie                         : std_logic; --tx interrupt enable
  signal txie_ce                      : std_logic; --only load txie on ping mem

-------------------------------------------------------------------------------
-- Component Declarations
-------------------------------------------------------------------------------
-- The following components are the building blocks of the tx state machine

component FDR
  port 
   (
    Q : out std_logic;
    C : in std_logic;
    D : in std_logic;
    R : in std_logic
    );
end component;

component FDS
  port 
   (
    Q : out std_logic;
    C : in std_logic;
    D : in std_logic;
    S : in std_logic
    );
end component;

component FDRE
  port
   (
    Q  : out std_logic;
    C  : in std_logic;
    CE : in std_logic;
    D  : in std_logic;
    R  : in std_logic
    );
end component;

begin

  Tx_done_irq      <= txDone and txie; --added tx_done_irq output for interrupt
  Tx_done          <= txDone; -- added txDone for ping pong control

  full_half_n      <= '1'when C_DUPLEX = 1 else '0'; 
  waitdone_pre_sfd <= phyCollision and not(full_half_n) and not(pre_sfd_zero);

  pre_sfd_done     <= pre_SFD_zero;
  phytx_en_i_n     <= not(phytx_en_i);
                 
  PRE_SFD_count: entity xps_ethernetlite_v2_00_c.cntr5bit
    port map
           (
            cntout  =>  open,
            clk     =>  clk, 
            rst     =>  rst,
            en      =>  txClkEn,
            ld      =>  phytx_en_i_n,
            load_in =>  "10011",
            zero    =>  pre_SFD_zero
            );  

  state_machine_rst <= rst;

  txNibbleCnt_is_1 <= not(txNibbleCnt(0)) and not(txNibbleCnt(1)) and not(txNibbleCnt(2))
     and not(txNibbleCnt(3)) and not(txNibbleCnt(4)) and not(txNibbleCnt(5)) and
     not(txNibbleCnt(6))
     and not(txNibbleCnt(7)) and not(txNibbleCnt(8)) and not(txNibbleCnt(9)) and
     not(txNibbleCnt(10))
     and txNibbleCnt(11);

  busFifoWrNibbleCnt_is_14 <= busFifoWrNibbleCnt(8) and busFifoWrNibbleCnt(9) 
    and busFifoWrNibbleCnt(10) and not(busFifoWrNibbleCnt(11));
  busFifoWrNibbleCnt_not_14 <= not(busFifoWrNibbleCnt_is_14);
    
  busFifoWrNibbleCnt_is_15 <= (busFifoWrNibbleCnt(8) and busFifoWrNibbleCnt(9)
    and busFifoWrNibbleCnt(10) and busFifoWrNibbleCnt(11)); 
  busFifoWrNibbleCnt_not_15 <= not(busFifoWrNibbleCnt_is_15);
  
  crcCnt_not_0 <= crcCnt(0) or crcCnt(1) or crcCnt(2) or crcCnt(3);      
  crcCnt_is_0 <= not crcCnt_not_0;  

  jamTxNibCnt_not_0 <= jamTxNibCnt(0) or jamTxNibCnt(1) or jamTxNibCnt(2) or 
    jamTxNibCnt(3);  
  jamTxNibCnt_is_0 <= not(jamTxNibCnt_not_0);
  
  colWindowNibCnt_not_0 <= colWindowNibCnt(0) or colWindowNibCnt(1) or 
    colWindowNibCnt(2) or colWindowNibCnt(3) or colWindowNibCnt(4) or 
    colWindowNibCnt(5) or colWindowNibCnt(6) or colWindowNibCnt(7); 
  colWindowNibCnt_is_0 <= not(colWindowNibCnt_not_0);
  
  colRetryCnt_is_15 <= not(colRetryCnt(0)) and colRetryCnt(1) and 
    colRetryCnt(2) and colRetryCnt(3) and colRetryCnt(4);    
-------------------------------------------------------------------------------
-- Transmit Interrupt Enable Register
-------------------------------------------------------------------------------
  TXIE_I: FDRE
  port map 
   (
    Q => txie              , --[out]
    C => clk               , --[in]
    CE => txie_ce          , --[in]
    D => tx_DPM_rd_data(0) , --[in]
    R => rst                 --[in]
    );

   txie_ce <= idle and not tx_pong_ping_l;

-------------------------------------------------------------------------------
-- idle state
-------------------------------------------------------------------------------    
  goto_idle <= txDonePause;
  
  stay_idle <= idle and not(tx_DPM_rd_data(3)); 
  
  idle_D <= goto_idle or stay_idle;

   state0a: FDS
     port map 
       (
       Q => idle,             --[out]
       C => clk,              --[in]
       D => idle_D,           --[in]
       S => state_machine_rst --[in]
      );
     
      Tx_idle <= idle;  
   -------------------------------------------------------------------------------
   -- txLngthRdNib1 state
   -------------------------------------------------------------------------------    
     goto_txLngthRdNib1_1 <= idle and tx_DPM_rd_data(3) and not(tx_DPM_rd_data(2));
   
     goto_txLngthRdNib1_2 <= retryReset;
    
     txLngthRdNib1_D      <= goto_txLngthRdNib1_1 or goto_txLngthRdNib1_2;
     
   state1a: FDR
     port map 
       (
       Q => txLngthRdNib1,    --[out]
       C => clk,              --[in]
       D => txLngthRdNib1_D,  --[in]
       R => state_machine_rst --[in]
      );
   -------------------------------------------------------------------------------
   -- txLngthRdNib2 state
   -------------------------------------------------------------------------------    
     goto_txLngthRdNib2 <= txLngthRdNib1;
   
     txLngthRdNib2_D    <= goto_txLngthRdNib2;
     
   state2a: FDR
     port map 
       (
       Q => txLngthRdNib2,    --[out]
       C => clk,              --[in]
       D => txLngthRdNib2_D,  --[in]
       R => state_machine_rst --[in]
       );  
   -------------------------------------------------------------------------------
   -- txLngthRdNib3 state
   -------------------------------------------------------------------------------    
     goto_txLngthRdNib3 <= txLngthRdNib2;
   
     txLngthRdNib3_D    <= goto_txLngthRdNib3;
     
   state3a: FDR
     port map 
      (
       Q => txLngthRdNib3,    --[out]
       C => clk,              --[in]
       D => txLngthRdNib3_D,  --[in]
       R => state_machine_rst --[in]
      ); 
   -------------------------------------------------------------------------------
   -- txLngthRdNib4 state
   -------------------------------------------------------------------------------    
     goto_txLngthRdNib4 <= txLngthRdNib3;
   
     txLngthRdNib4_D    <= goto_txLngthRdNib4;
     
   state4a: FDR
     port map 
      (
       Q => txLngthRdNib4,    --[out]
       C => clk,              --[in]
       D => txLngthRdNib4_D,  --[in]
       R => state_machine_rst --[in]
      );   
   -------------------------------------------------------------------------------
   -- lngthdelay1 state
   -------------------------------------------------------------------------------    
     goto_lngthdelay1 <= txLngthRdNib4;
   
     lngthdelay1_D    <= goto_lngthdelay1;
     
   state5a: FDR
     port map 
      (
       Q => lngthdelay1,      --[out]
       C => clk,              --[in]
       D => lngthdelay1_D,    --[in]
       R => state_machine_rst --[in]
      );    
   -------------------------------------------------------------------------------
   -- lngthdelay1 state
   -------------------------------------------------------------------------------    
     goto_lngthdelay2 <= lngthdelay1;
   
     lngthdelay2_D    <= goto_lngthdelay2;
     
   state6a: FDR
     port map 
      (
       Q => lngthdelay2,      --[out]
       C => clk,              --[in]
       D => lngthdelay2_D,    --[in]
       R => state_machine_rst --[in]
      );    
           
   -------------------------------------------------------------------------------
   -- ldLngthCntr state
   -------------------------------------------------------------------------------    
     goto_ldLngthCntr <= lngthdelay2;
     
     stay_ldLngthCntr <= ldLngthCntr and deferring;
   
     ldLngthCntr_D    <= goto_ldLngthCntr or stay_ldLngthCntr;
     
   state7a: FDR
     port map 
      (
       Q => ldLngthCntr,      --[out]
       C => clk,              --[in]
       D => ldLngthCntr_D,    --[in]
       R => state_machine_rst --[in]
      );    
     
   -------------------------------------------------------------------------------
   -- preamble state
   -------------------------------------------------------------------------------      
     goto_preamble <= (ldLngthCntr and (not(deferring)));
       
     stay_preamble <= preamble and busFifoWrNibbleCnt_not_14;
     
     preamble_D    <= goto_preamble or stay_preamble;
   
   state8a: FDR
     port map 
      (
       Q => preamble,         --[out]
       C => clk,              --[in]
       D => preamble_D,       --[in]
       R => state_machine_rst --[in]
      );  
   -------------------------------------------------------------------------------
   -- checkBusFifoFullSFD state
   -------------------------------------------------------------------------------      
     goto_checkBusFifoFullSFD <= preamble and busFifoWrNibbleCnt_is_14;
     
     stay_checkBusFifoFullSFD <= checkBusFifoFullSFD and busFifoFull;
     
     checkBusFifoFullSFD_D    <= goto_checkBusFifoFullSFD 
                                 or stay_checkBusFifoFullSFD;
     
   state9a: FDR
     port map 
      (
       Q => checkBusFifoFullSFD,   --[out]
       C => clk,                   --[in]
       D => checkBusFifoFullSFD_D, --[in]
       R => state_machine_rst      --[in]
      );
   -------------------------------------------------------------------------------
   -- SFD state
   -------------------------------------------------------------------------------
     goto_SFD <= checkBusFifoFullSFD and not (busFifoFull);
     
     stay_SFD <= SFD and busFifoWrNibbleCnt_not_15;
     
     SFD_D    <= goto_SFD or stay_SFD;
     
   state10a: FDR
     port map 
      (
       Q => SFD,              --[out]
       C => clk,              --[in]
       D => SFD_D,            --[in]
       R => state_machine_rst --[in]
       );
   -------------------------------------------------------------------------------
   -- checkBusFifoFull state
   -------------------------------------------------------------------------------   
     goto_checkBusFifoFull_1 <= loadBusFifo and not(goto_checkCrc) and
                                not(goto_checkBusFifoFullJam_1);
     
     goto_checkBusFifoFull_2 <= SFD and busFifoWrNibbleCnt_is_15;
       
     stay_checkBusFifoFull   <= checkBusFifoFull and busFifoFull and
                              not (goto_checkBusFifoFullJam_1);
     
     checkBusFifoFull_D      <= goto_checkBusFifoFull_1 or goto_checkBusFifoFull_2 
                                or stay_checkBusFifoFull;
   
   state11a: FDR
     port map 
      (
       Q => checkBusFifoFull,   --[out]
       C => clk,                --[in]
       D => checkBusFifoFull_D, --[in]
       R => state_machine_rst   --[in]
       );    
   -------------------------------------------------------------------------------
   -- loadBusFifo state
   -------------------------------------------------------------------------------   
     goto_loadBusFifo <= checkBusFifoFull and not(busFifoFull) and 
                         not(goto_checkCrc) and not(goto_checkBusFifoFullJam_1);
     
     loadBusFifo_D    <= goto_loadBusFifo;
   
   state12a: FDR
     port map 
      (
       Q => loadBusFifo,      --[out]
       C => clk,              --[in]
       D => loadBusFifo_D,    --[in]
       R => state_machine_rst --[in]
       );  
   -------------------------------------------------------------------------------
   -- checkCrc state
   -------------------------------------------------------------------------------    
     goto_checkCrc <= loadBusFifo and txNibbleCnt_is_1 and 
                      not(goto_checkBusFifoFullJam_1);
       
     checkCrc_D    <= goto_checkCrc;
     
   state13a: FDR
     port map 
      (
       Q => checkCrc,         --[out]
       C => clk,              --[in]
       D => checkCrc_D,       --[in]
       R => state_machine_rst --[in]
      );      
   -------------------------------------------------------------------------------
   -- checkBusFifoFullCrc state
   -------------------------------------------------------------------------------        
     goto_checkBusFifoFullCrc_1 <=  checkCrc and not(goto_checkBusFifoFullJam_1);
     
     goto_checkBusFifoFullCrc_2 <= loadBusFifoCrc and not(goto_checkBusFifoFullJam_1);
     
     stay_checkBusFifoFullCrc   <=  checkBusFifoFullCrc and busFifoFull and
                                  not(goto_checkBusFifoFullJam_1);
     
     checkBusFifoFullCrc_D      <= goto_checkBusFifoFullCrc_1 or 
                                   goto_checkBusFifoFullCrc_2 or stay_checkBusFifoFullCrc;  
     
   state14a: FDR
     port map 
      (
       Q => checkBusFifoFullCrc,   --[out]
       C => clk,                   --[in]
       D => checkBusFifoFullCrc_D, --[in]
       R => state_machine_rst      --[in]
      );  
   -------------------------------------------------------------------------------
   -- loadBusFifoCrc state
   -------------------------------------------------------------------------------      
     goto_loadBusFifoCrc_1 <= checkBusFifoFullCrc and not(busFifoFull) and
                              crcCnt_not_0 and not(goto_checkBusFifoFullJam_1);
       
     loadBusFifoCrc_D      <= goto_loadBusFifoCrc_1;  
     
   state15a: FDR
     port map 
      (
       Q => loadBusFifoCrc,   --[out]
       C => clk,              --[in]
       D => loadBusFifoCrc_D, --[in]
       R => state_machine_rst --[in]
      );  
   -------------------------------------------------------------------------------
   -- waitFifoEmpty state
   -------------------------------------------------------------------------------       
     
     goto_waitFifoEmpty_2 <= checkBusFifoFullCrc and crcCnt_is_0 and
                             not(busFifoFull) and not(goto_checkBusFifoFullJam_1);
     
     stay_waitFifoEmpty   <= waitFifoEmpty and not(busFifoEmpty) and
                             not(goto_checkBusFifoFullJam_1);
     
     waitFifoEmpty_D      <= goto_waitFifoEmpty_2 or stay_waitFifoEmpty;
     
   state16a: FDR
     port map 
      (
       Q => waitFifoEmpty,    --[out]
       C => clk,              --[in]
       D => waitFifoEmpty_D,  --[in]
       R => state_machine_rst --[in]
      );  
   -------------------------------------------------------------------------------
   -- txDone state
   -------------------------------------------------------------------------------   
     goto_txDone_1 <= waitFifoEmpty and busFifoEmpty and 
                      not(goto_checkBusFifoFullJam_1);
     
     goto_txDone_2 <= half_dup_error or chgMacAdr14;
     
     txDone_D      <= goto_txDone_1 or goto_txDone_2;
     
   state17a: FDR
     port map 
      (
       Q => txDone,           --[out]
       C => clk,              --[in]
       D => txDone_D,         --[in]
       R => state_machine_rst --[in]
      );  
   -------------------------------------------------------------------------------
   -- checkBusFifoFullJam state
   -------------------------------------------------------------------------------    
     goto_checkBusFifoFullJam_1 <= (checkBusFifoFull or loadBusFifo or checkCrc or
                                    checkBusFifoFullCrc or waitFifoEmpty) and
                                    phyCollision and not(full_half_n);
     
     goto_checkBusFifoFullJam_2 <= loadBusFifoJam;
       
     stay_checkBusFifoFullJam   <= checkBusFifoFullJam and (busFifoFull or
                                 not(pre_SFD_zero));
     
     checkBusFifoFullJam_D      <= goto_checkBusFifoFullJam_1 or
                                   goto_checkBusFifoFullJam_2 or stay_checkBusFifoFullJam;
     
   state18a: FDR
     port map 
      (
       Q => checkBusFifoFullJam,   --[out]
       C => clk,                   --[in]
       D => checkBusFifoFullJam_D, --[in]
       R => state_machine_rst      --[in]
       );  
   -------------------------------------------------------------------------------
   -- loadBusFifoJam state
   -------------------------------------------------------------------------------    
     goto_loadBusFifoJam <= checkBusFifoFullJam and not(stay_checkBusFifoFullJam) and 
                            jamTxNibCnt_not_0;
     
     loadBusFifoJam_D    <= goto_loadBusFifoJam;
     
   state19a: FDR
     port map 
      (
       Q => loadBusFifoJam,   --[out]
       C => clk,              --[in]
       D => loadBusFifoJam_D, --[in]
       R => state_machine_rst --[in]
      );  
   -------------------------------------------------------------------------------
   -- half_dup_error state
   -------------------------------------------------------------------------------  
     goto_half_dup_error_1 <= checkBusFifoFullJam and not(busFifoFull or
                              not(pre_SFD_zero)) and jamTxNibCnt_is_0 and
                              colWindowNibCnt_not_0 and colRetryCnt_is_15;
       
     goto_half_dup_error_2 <= checkBusFifoFullJam and not(busFifoFull or
                              not(pre_SFD_zero)) and jamTxNibCnt_is_0 and 
                              colWindowNibCnt_is_0;
       
     half_dup_error_D      <= goto_half_dup_error_1 or goto_half_dup_error_2;
    
   state20a: FDR
     port map 
      (
       Q => half_dup_error,   --[out]
       C => clk,              --[in]
       D => half_dup_error_D, --[in]
       R => state_machine_rst --[in]
      );
   -------------------------------------------------------------------------------
   -- collisionRetry state
   -------------------------------------------------------------------------------   
     goto_collisionRetry <= checkBusFifoFullJam and not(stay_checkBusFifoFullJam) 
                            and not(goto_half_dup_error_1) and 
                            not(goto_half_dup_error_2) and
                            not(goto_loadBusFifoJam);
       
     collisionRetry_D    <= goto_collisionRetry;
   
   state21a: FDR
     port map 
      (
       Q => collisionRetry,   --[out]
       C => clk,              --[in]
       D => collisionRetry_D, --[in]
       R => state_machine_rst --[in]
      );  
   -------------------------------------------------------------------------------
   -- retryWaitFifoEmpty state
   -------------------------------------------------------------------------------   
     goto_retryWaitFifoEmpty <= collisionRetry;
   
     stay_retryWaitFifoEmpty <= retryWaitFifoEmpty and not(busFifoEmpty);
       
     retryWaitFifoEmpty_D    <= goto_retryWaitFifoEmpty or stay_retryWaitFifoEmpty;
    
   state22a: FDR
     port map 
      (
       Q => retryWaitFifoEmpty,   --[out]
       C => clk,                  --[in]
       D => retryWaitFifoEmpty_D, --[in]
       R => state_machine_rst     --[in]
      );
   -------------------------------------------------------------------------------
   -- retryReset state
   -------------------------------------------------------------------------------   
     goto_retryReset <= retryWaitFifoEmpty and busFifoEmpty;
       
     retryReset_D    <= goto_retryReset;
   
   state23a: FDR
     port map 
      (
       Q => retryReset,       --[out]
       C => clk,              --[in]
       D => retryReset_D,     --[in]
       R => state_machine_rst --[in]
      );  
   -------------------------------------------------------------------------------
   -- txDone2 state
   -------------------------------------------------------------------------------   
     goto_txDone2 <= txDone;
     
     txDone2_D    <= goto_txDone2;
     
   state24a: FDR
     port map 
      (
       Q => txDone2,          --[out]
       C => clk,              --[in]
       D => txDone2_D,        --[in]
       R => state_machine_rst --[in]
      );  
   -------------------------------------------------------------------------------
   -- txDonePause state
   -------------------------------------------------------------------------------   
     goto_txDonePause <= txDone2;
     
     txDonePause_D    <= goto_txDonePause;
     
   state25a: FDR
     port map 
      (
       Q => txDonePause,      --[out]
       C => clk,              --[in]
       D => txDonePause_D,    --[in]
       R => state_machine_rst --[in]
      );    
   -------------------------------------------------------------------------------
   -- chgMacAdr1 state
   -------------------------------------------------------------------------------    
     goto_chgMacAdr1 <= idle and tx_DPM_rd_data(3) and tx_DPM_rd_data(2);
   
     chgMacAdr1_D    <= goto_chgMacAdr1 ;
     
   state26a: FDR
     port map 
      (
       Q => chgMacAdr1,       --[out]
       C => clk,              --[in]
       D => chgMacAdr1_D,     --[in]
       R => state_machine_rst --[in]
      );  
   -------------------------------------------------------------------------------
   -- chgMacAdr2 state
   -------------------------------------------------------------------------------    
     goto_chgMacAdr2 <= chgMacAdr1;
   
     chgMacAdr2_D    <= goto_chgMacAdr2 ;
     
   state27a: FDR
     port map 
      (
       Q => chgMacAdr2,       --[out]
       C => clk,              --[in]
       D => chgMacAdr2_D,     --[in]
       R => state_machine_rst --[in]
      );  
   -------------------------------------------------------------------------------
   -- chgMacAdr3 state
   -------------------------------------------------------------------------------    
     goto_chgMacAdr3 <= chgMacAdr2;
   
     chgMacAdr3_D    <= goto_chgMacAdr3 ;
     
   state28a: FDR
     port map 
      (
       Q => chgMacAdr3,       --[out]
       C => clk,              --[in]
       D => chgMacAdr3_D,     --[in]
       R => state_machine_rst --[in]
      );    
   -------------------------------------------------------------------------------
   -- chgMacAdr4 state
   -------------------------------------------------------------------------------    
     goto_chgMacAdr4 <= chgMacAdr3;
   
     chgMacAdr4_D    <= goto_chgMacAdr4 ;
     
   state29a: FDR
     port map 
      (
       Q => chgMacAdr4,       --[out]
       C => clk,              --[in]
       D => chgMacAdr4_D,     --[in]
       R => state_machine_rst --[in]
       );  
   -------------------------------------------------------------------------------
   -- chgMacAdr5 state
   -------------------------------------------------------------------------------    
     goto_chgMacAdr5 <= chgMacAdr4;
   
     chgMacAdr5_D    <= goto_chgMacAdr5 ;
     
   state30a: FDR
     port map 
      (
       Q => chgMacAdr5,       --[out]
       C => clk,              --[in]
       D => chgMacAdr5_D,     --[in]
       R => state_machine_rst --[in]
      );   
   -------------------------------------------------------------------------------
   -- chgMacAdr6 state
   -------------------------------------------------------------------------------    
     goto_chgMacAdr6 <= chgMacAdr5;
   
     chgMacAdr6_D    <= goto_chgMacAdr6 ;
     
   state31a: FDR
     port map 
      (
       Q => chgMacAdr6,       --[out]
       C => clk,              --[in]
       D => chgMacAdr6_D,     --[in]
       R => state_machine_rst --[in]
      );  
   -------------------------------------------------------------------------------
   -- chgMacAdr7 state
   -------------------------------------------------------------------------------    
     goto_chgMacAdr7 <= chgMacAdr6;
   
     chgMacAdr7_D    <= goto_chgMacAdr7 ;
     
   state32a: FDR
     port map 
      (
       Q => chgMacAdr7,       --[out]
       C => clk,              --[in]
       D => chgMacAdr7_D,     --[in]
       R => state_machine_rst --[in]
      );    
   -------------------------------------------------------------------------------
   -- chgMacAdr8 state
   -------------------------------------------------------------------------------    
     goto_chgMacAdr8 <= chgMacAdr7;
   
     chgMacAdr8_D    <= goto_chgMacAdr8 ;
     
   state33a: FDR
     port map 
      (
       Q => chgMacAdr8,       --[out]
       C => clk,              --[in]
       D => chgMacAdr8_D,     --[in]
       R => state_machine_rst --[in]
      );  
   -------------------------------------------------------------------------------
   -- chgMacAdr9 state
   -------------------------------------------------------------------------------    
     goto_chgMacAdr9 <= chgMacAdr8;
   
     chgMacAdr9_D    <= goto_chgMacAdr9 ;
     
   state34a: FDR
     port map 
      (
       Q => chgMacAdr9,       --[out]
       C => clk,              --[in]
       D => chgMacAdr9_D,     --[in]
       R => state_machine_rst --[in]
      );     
   -------------------------------------------------------------------------------
   -- chgMacAdr10 state
   -------------------------------------------------------------------------------    
     goto_chgMacAdr10 <= chgMacAdr9;
   
     chgMacAdr10_D    <= goto_chgMacAdr10 ;
     
   state35a: FDR
     port map 
      (
       Q => chgMacAdr10,      --[out]
       C => clk,              --[in]
       D => chgMacAdr10_D,    --[in]
       R => state_machine_rst --[in]
      );   
   -------------------------------------------------------------------------------
   -- chgMacAdr11 state
   -------------------------------------------------------------------------------    
     goto_chgMacAdr11 <= chgMacAdr10;
   
     chgMacAdr11_D    <= goto_chgMacAdr11 ;
     
   state36a: FDR
     port map 
      (
       Q => chgMacAdr11,      --[out]
       C => clk,              --[in]
       D => chgMacAdr11_D,    --[in]
       R => state_machine_rst --[in]
      );  
   -------------------------------------------------------------------------------
   -- chgMacAdr12 state
   -------------------------------------------------------------------------------    
     goto_chgMacAdr12 <= chgMacAdr11;
   
     chgMacAdr12_D    <= goto_chgMacAdr12 ;
     
   state37a: FDR
     port map 
      (
       Q => chgMacAdr12,      --[out]
       C => clk,              --[in]
       D => chgMacAdr12_D,    --[in]
       R => state_machine_rst --[in]
      );    
   -------------------------------------------------------------------------------
   -- chgMacAdr13 state
   -------------------------------------------------------------------------------    
     goto_chgMacAdr13 <= chgMacAdr12;
   
     chgMacAdr13_D    <= goto_chgMacAdr13 ;
     
   state38a: FDR
     port map 
      (
       Q => chgMacAdr13,      --[out]
       C => clk,              --[in]
       D => chgMacAdr13_D,    --[in]
       R => state_machine_rst --[in]
      );  
   -------------------------------------------------------------------------------
   -- chgMacAdr14 state
   -------------------------------------------------------------------------------    
     goto_chgMacAdr14 <= chgMacAdr13;
   
     chgMacAdr14_D    <= goto_chgMacAdr14 ;
     
   state39a: FDR
     port map 
      (
       Q => chgMacAdr14,      --[out]
       C => clk,              --[in]
       D => chgMacAdr14_D,    --[in]
       R => state_machine_rst --[in]
      );    
-------------------------------------------------------------------------------   
-- end of states
-------------------------------------------------------------------------------   

  REG_PROCESS : process (clk)
  begin  --
    if (clk'event and clk = '1') then     -- rising clock edge
      if (rst = '1') then
        phytx_en_reg        <= '0';
        busFifoWrCntRst_reg <= '0';
        retrying_reg        <= '0';
        txCrcEn_reg         <= '0';   
      else

        phytx_en_reg        <= phytx_en_i;
        busFifoWrCntRst_reg <= busFifoWrCntRst_i;
        retrying_reg        <= retrying_i;
        txCrcEn_reg         <= txCrcEn_i;          
      end if;
    end if;
  end process REG_PROCESS;
 
 b : process (phytx_en_reg,busFifoWrCntRst_reg,retrying_reg,txCrcEn_reg,
               txDone, txLngthRdNib1, idle, preamble, half_dup_error,
               collisionRetry, checkBusFifoFull, CheckBusFifoFullCRC,sfd,
               loadBusFifoCrc, checkBusFifoFullSFD)
  begin
    
    if (txDone='1' or txLngthRdNib1='1' or idle='1') then
      phytx_en_i <= '0';
    elsif (preamble = '1') then
      phytx_en_i <= '1';
    else
      phytx_en_i <= phytx_en_reg;
    end if;
    
    if (half_dup_error='1' or txDone='1' or txLngthRdNib1='1' or idle='1') then
      busFifoWrCntRst_i <= '1';
    elsif (preamble = '1') then
      busFifoWrCntRst_i <= '0';
    else
      busFifoWrCntRst_i <= busFifoWrCntRst_reg;
    end if;    
    
    if (collisionRetry='1') then
      retrying_i <= '1';
    elsif (idle = '1') then
      retrying_i <= '0';
    else
      retrying_i <= retrying_reg;
    end if;
    
    if (checkBusFifoFull='1') then
      txCrcEn_i <= '1';
    elsif (checkBusFifoFullSFD='1' or CheckBusFifoFullCRC='1' or sfd='1' or
           idle='1' or loadBusFifoCrc='1' or preamble='1' or 
           txLngthRdNib1='1') then
      txCrcEn_i <= '0';
    else
      txCrcEn_i <= txCrcEn_reg;
    end if;
  end process b;
   
  FSMD : process(crcCnt_is_0,jamTxNibCnt,goto_checkBusFifoFullCrc_1,
              pre_SFD_zero, checkBusFifoFullJam, full_half_n,retryReset,
              txDonePause,loadBusFifo, loadBusFifoJam, checkCrc, txLngthRdNib2,
              txLngthRdNib3, txLngthRdNib4, txDone2, chgMacAdr2, chgMacAdr3,
              chgMacAdr4, chgMacAdr5, chgMacAdr6, chgMacAdr7, chgMacAdr8,
              chgMacAdr9, chgMacAdr10, chgMacAdr11, chgMacAdr12, chgMacAdr13,
              chgMacAdr14, chgMacAdr1, lngthDelay1, lngthDelay2, idle,
              checkBusFifoFull, txDone, ldLngthCntr,txie,half_dup_error,
              collisionRetry, checkBusFifoFullCrc, loadBusFifoCrc, preamble, 
              SFD, txLngthRdNib1 )
  
  begin

   if (checkBusFifoFullJam='1' and pre_SFD_zero = '1' and full_half_n = '0' and
       (jamTxNibCnt = "0111")) then
     jam_rst <= '1';
   else
     jam_rst <= '0';     
   end if;
   
   busFifoWrCntEn      <= '1'; -- temp 
   
   if (half_dup_error='1') then
     txLateColnRst <= '1';
   else
     txLateColnRst <= '0';
   end if;    
   
   if (half_dup_error='1') then
     txExcessDefrlRst <= '1';     
   else
     txExcessDefrlRst <= '0';        
   end if;
   
   if (collisionRetry = '1') then
     initBackoff <= '1';
     txColRetryCntEnbl <= '1';
   else
     initBackoff <= '0';
     txColRetryCntEnbl <= '0';
   end if;

   if (retryReset = '1') or
      (txDonePause = '1') then -- clear up any built up garbage in async FIFOs 
                               -- at the end of a packet
     txRetryRst <= '1';        
   else
     txRetryRst <= '0';     
   end if;    
   
   if (idle = '1') then
     txNibbleCntRst_i <= '1';
   else
     txNibbleCntRst_i <= '0';
   end if;
   
   if (idle = '1') then
     txColRetryCntRst_n   <= '0';
   else
     txColRetryCntRst_n   <= '1';
   end if;
   
   if ((checkBusFifoFullCrc = '1') or (loadBusFifoCrc = '1')) then
     txCrcShftOutEn   <= '1';
   else
     txCrcShftOutEn   <= '0';
   end if;
   
   if (preamble = '1') then
     enblPre   <= '1';
   else
     enblPre   <= '0';
   end if;
   
   if (SFD = '1') then
     enblSFD   <= '1';
   else
     enblSFD   <= '0';
   end if;
   
   if (loadBusFifo = '1') then
     enblData <= '1';
   else
     enblData <= '0';
   end if;   
   
   if (loadBusFifoCrc = '1') then
     enblCrc <= '1';
   else
     enblCrc <= '0';
   end if;
  
   if (sfd = '1') then
     txNibbleCntLd_i   <= '1';
   else
     txNibbleCntLd_i   <= '0';
   end if;
  
  if (checkBusFifoFullCrc = '1' and crcCnt_is_0  = '1') or
      ((checkBusFifoFullJam='1' or loadBusFifoJam='1') 
       and pre_SFD_zero = '1' and full_half_n = '0') or
     (collisionRetry = '1' ) or  (half_dup_error = '1') or
     (checkCrc = '1' and goto_checkBusFifoFullCrc_1 = '0') then
     enblclear <= '1';
   else
     enblclear <= '0';
   end if;
  
   if ((loadBusFifo = '1') or
       (preamble = '1') or
       (sfd = '1') or
       (loadBusFifoCrc = '1')
       ) then
     busFifoWr_i   <= '1';
   else
     busFifoWr_i   <= '0';
   end if;
   
   if (loadBusFifo = '1') then
     txEnNibbleCnt_i   <= '1';
   else
     txEnNibbleCnt_i   <= '0';
   end if;   
   
   if (loadBusFifoJam = '1') then
     JamTxNibCntEnbl   <= '1';
   else
     JamTxNibCntEnbl   <= '0';
   end if;  
   
   if (idle = '1') or
      (txLngthRdNib1 = '1') or
      (txLngthRdNib2 = '1') or
      (txLngthRdNib3 = '1') or
      (txLngthRdNib4 = '1') or
      (txDone = '1') or
      (txDone2 = '1') or
      (txDonePause = '1') then
     tx_addr_load <= '1'; --enable address constants
   else
     tx_addr_load <= '0';
   end if;
   
   if (loadBusFifo = '1') or
      (chgMacAdr2 = '1') or
      (chgMacAdr3 = '1') or
      (chgMacAdr4 = '1') or
      (chgMacAdr5 = '1') or
      (chgMacAdr6 = '1') or
      (chgMacAdr7 = '1') or
      (chgMacAdr8 = '1') or
      (chgMacAdr9 = '1') or
      (chgMacAdr10 = '1') or
      (chgMacAdr11 = '1') or
      (chgMacAdr12 = '1') or
      (chgMacAdr13 = '1') or
      (chgMacAdr14 = '1') then
     tx_addr_en <= '1'; --enable address increment
   else
     tx_addr_en <= '0';
   end if;   
   
   if (preamble = '1') or
      (chgMacAdr1 = '1') then
     tx_start <= '1'; -- reset address to 0 for start of transmit
   else
     tx_start <= '0';
   end if;
   
   if (txLngthRdNib1 = '1') then
     enbl_txLength_nib1 <= '1';
   else
     enbl_txLength_nib1 <= '0';
   end if;
   
   if (txLngthRdNib2 = '1') then
     enbl_txLength_nib2 <= '1';
   else
     enbl_txLength_nib2 <= '0';
   end if;
     
   if (txLngthRdNib3 = '1') then
     enbl_txLength_nib3 <= '1';
   else
     enbl_txLength_nib3 <= '0';
   end if;
     
   if (txLngthRdNib4 = '1') then
     enbl_txLength_nib4 <= '1';
   else
     enbl_txLength_nib4 <= '0';
   end if;
   
   if (lngthDelay1 = '1') then
     lengthDelay1 <= '1';
   else
     lengthDelay1 <= '0';
   end if;
   
   if (lngthDelay2 = '1') then
     lengthDelay2 <= '1';
   else
     lengthDelay2 <= '0';
   end if;
      
   if (idle = '1') or
      (txLngthRdNib1 = '1') or
      (txLngthRdNib2 = '1') or
      (txLngthRdNib3 = '1') or
      (txLngthRdNib4 = '1') or
      (lngthDelay1 = '1') or
      (lngthDelay2 = '1') or 
      (checkBusFifoFull = '1') or
      (txDone = '1') or
      (txDone2 = '1') or
      (txDonePause = '1') or
      (ldLngthCntr = '1') or
      (chgMacAdr1 = '1') or
      (chgMacAdr2 = '1') or
      (chgMacAdr3 = '1') or
      (chgMacAdr4 = '1') or
      (chgMacAdr5 = '1') or
      (chgMacAdr6 = '1') or
      (chgMacAdr7 = '1') or
      (chgMacAdr8 = '1') or
      (chgMacAdr9 = '1') or
      (chgMacAdr10 = '1') or
      (chgMacAdr11 = '1') or
      (chgMacAdr12 = '1') or
      (chgMacAdr13 = '1') or
      (chgMacAdr14 = '1') then
     tx_DPM_ce <= '1';
   else
     tx_DPM_ce <= '0';
   end if;
     
   if (txie = '0') then
      if (idle = '1') then
        tx_DPM_wr_data <= x"0";
      elsif (txDone = '1' or txDone2 = '1') then
        tx_DPM_wr_data <= x"0";
      else
        tx_DPM_wr_data <= x"0";
      end if;
   else
      if (idle = '1') then
        tx_DPM_wr_data <= x"8";
      elsif (txDone = '1' or txDone2 = '1') then
        tx_DPM_wr_data <= x"8";
      else
        tx_DPM_wr_data <= x"8";
      end if;
   end if;
     
   if (idle = '1') then
     tx_DPM_wr_rd_n <= '0';
   elsif (txDone = '1' or txDone2 = '1') then
     tx_DPM_wr_rd_n <= '1';   
   else   
     tx_DPM_wr_rd_n <= '0';
   end if;
   
   if (loadBusFifoJam = '1') then
     enblJam <= '1';
   else
     enblJam <= '0';
   end if;
                                    
  end process FSMD;


OUTPUT_REG1:process (clk)
 
 begin
 
 if (clk'event and clk='1') then
      if (rst = '1') then
             mac_addr_ram_we <= '0';
      
      else

          if (idle_D = '1') then
            mac_addr_ram_we <= '0';
          elsif (chgMacAdr3_D = '1') or 
               (chgMacAdr4_D = '1') or
               (chgMacAdr5_D = '1') or
               (chgMacAdr6_D = '1') or
               (chgMacAdr7_D = '1') or
               (chgMacAdr8_D = '1') or
               (chgMacAdr9_D = '1') or
               (chgMacAdr10_D = '1') or
               (chgMacAdr11_D = '1') or
               (chgMacAdr12_D = '1') or
               (chgMacAdr13_D = '1') or
               (chgMacAdr14_D = '1') then
             mac_addr_ram_we <= '1';     
          else
             mac_addr_ram_we <= '0';
         end if;
      end if;
  end if;
end process OUTPUT_REG1;

OUTPUT_REG2:process (clk)
 
 begin
 
 if (clk'event and clk='1') then
      if (rst = '1') then
             mac_addr_ram_addr_wr <= x"0";
      
      else
  
           if idle_D = '1' then
                mac_addr_ram_addr_wr <= x"0";
      
           elsif chgMacAdr3_D = '1' then
                mac_addr_ram_addr_wr <= x"0";             
  
           elsif chgMacAdr4_D = '1' then
                mac_addr_ram_addr_wr <= x"1";             
  
           elsif chgMacAdr5_D = '1' then
                mac_addr_ram_addr_wr <= x"2";             
  
           elsif chgMacAdr6_D = '1' then
                mac_addr_ram_addr_wr <= x"3";             
  
           elsif chgMacAdr7_D = '1' then
                mac_addr_ram_addr_wr <= x"4";             
  
           elsif chgMacAdr8_D = '1' then
                mac_addr_ram_addr_wr <= x"5";             
  
           elsif chgMacAdr9_D = '1' then
                mac_addr_ram_addr_wr <= x"6";             
  
           elsif chgMacAdr10_D = '1' then
                mac_addr_ram_addr_wr <= x"7";             
  
           elsif chgMacAdr11_D = '1' then
                mac_addr_ram_addr_wr <= x"8";             
  
           elsif chgMacAdr12_D = '1' then
                mac_addr_ram_addr_wr <= x"9";             
  
           elsif chgMacAdr13_D = '1' then
                mac_addr_ram_addr_wr <= x"a";             
    
           elsif chgMacAdr14_D = '1' then
                mac_addr_ram_addr_wr <= x"b";             
  
           else
               mac_addr_ram_addr_wr <= x"0";
           
           end if;
     end if;
     
   end if;
   
 end process OUTPUT_REG2;
    
   txNibbleCntRst      <= txNibbleCntRst_i;       
   txEnNibbleCnt       <= txEnNibbleCnt_i;
   txNibbleCntLd       <= txNibbleCntLd_i;
   
   busFifoWr           <= busFifoWr_i;
   phytx_en            <= phytx_en_i;
   txCrcEn             <= txCrcEn_i;

   busFifoWrCntRst     <= busFifoWrCntRst_i;
   retrying            <= retrying_i; 
 end implementation;
 
